/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
/*
 * Created on Feb 27, 2005
 */
package javax.tools.diagnostics.image;

import java.util.List;

/**
 * This class represents a single Address Space within the image. 
 * On some operating systems (e.g. z/OS), there can be more than 
 * one Address Space per core file (but generally with only one 
 * process per ImageAddressSpace). 
 *
 */
public interface ImageAddressSpace {

    /**
     * Get the process within this address space which caused the image to be created.
     * @return the process within this address space which caused 
     * the image to be created, if any. Return null if no individual 
     * process triggered the creation of the image.
     */
    ImageProcess getCurrentProcess();
    
    /**
     * Get the set of processes within the address space.
     * @return an iterator which provides all of the processes
     * within a given address space. In most images, there will 
     * only be one process within an ImageAddressSpace
     * 
     * @see ImageProcess
     */
    List<ImageProcess> getProcesses();
    
    /**
     * A factory method for creating pointers into this address space
     * @param address the address to point to
     * @return an ImagePointer for the specified address
     */
    ImagePointer getPointer(long address);
    
    /**
     * Get the raw memory in the address space.
     * 
     * @return An iterator of all the ImageSections in the address space.  Their union will be the total process address space
     * @see ImageSection
     */
    List<ImageSection> getImageSections();
}
