/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package javax.tools.diagnostics.image;

/**
 * Represents the state of a CPU or FPU register
 */
public interface ImageRegister {

    /**
     * Fetch the name of a register.
     * 
     * On some CPUs registers may have more than one conventional name. Recommended 
     * names for some CPUs are provided in the user guide. 
     *  
     * @return the conventional name of the register
     */
    String getName();
    
    /**
     * Get the value for the register.
     * @return an integral or floating point type which contains
     * the value for the register. The returned value may be an
     * instance of any subclass of Number. For instance, on x86
     * architectures with MMX, the XMM registers will be returned
     * as BigInteger instances
     * @throws CorruptDataException 
     */
    Number getValue() throws CorruptDataException;
    
}
