/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package javax.tools.diagnostics.image;

/**
 * Indicates that an attempt was made to access memory which is not included within the image
 */
public class MemoryAccessException extends DiagnosticException {
    
	private static final long serialVersionUID = 8883829684484518525L;

	private ImagePointer badPointer;
    
    /**
     * Build exception for the given location and description 
     * @param badPointer		the location which caused the access exception
     * @param description		text description
     */
    public MemoryAccessException(ImagePointer badPointer, String description) {
        super(description);
        this.badPointer = badPointer;
    }
    
    /**
     * Build exception for the given location and description 
     * @param badPointer		the location which caused the access exception
     */
    public MemoryAccessException(ImagePointer badPointer) {
        super();
        this.badPointer = badPointer;
    }
    
    /**
     * Get a pointer into the image where the access failed.
     * @return The pointer into the image where the access failed
     */
    public ImagePointer getPointer() {
    	return badPointer;
    }
}
