/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/

package org.apache.kato.jvmti.javaruntime;

import java.io.File;
import java.io.IOException;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.logging.Level;

import javax.tools.diagnostics.image.CorruptDataException;
import javax.tools.diagnostics.image.DataUnavailable;
import javax.tools.diagnostics.image.ImagePointer;
import javax.tools.diagnostics.image.MemoryAccessException;
import javax.tools.diagnostics.runtime.java.JavaClassLoader;
import javax.tools.diagnostics.runtime.java.JavaHeap;
import javax.tools.diagnostics.runtime.java.JavaMethod;
import javax.tools.diagnostics.runtime.java.JavaMonitor;
import javax.tools.diagnostics.runtime.java.JavaObject;
import javax.tools.diagnostics.runtime.java.JavaReference;
import javax.tools.diagnostics.runtime.java.JavaRuntime;
import javax.tools.diagnostics.runtime.java.JavaThread;
import javax.tools.diagnostics.runtime.java.JavaVMInitArgs;

import org.apache.kato.jvmti.javaruntime.model.Model;
import org.apache.kato.jvmti.javaruntime.model.SimpleImagePointer;
import org.apache.kato.jvmti.reader.CJVMTIBinReader;
import org.apache.kato.jvmti.reader.CLogger;

public class JavaRuntimeImpl implements JavaRuntime {	
	private Model model=null;
	private File source;
	
	public JavaRuntimeImpl(Model model, File source) throws IOException {
		this.model=model;
		this.source = source;
	}
	
	@Override
	public List<JavaMethod> getCompiledMethods() {
		return Collections.emptyList();
	}

	@Override
	public List<JavaReference> getHeapRoots() {
		return Collections.emptyList();
	}

	private List<JavaHeap> heap = null;
	@Override
	public List<JavaHeap> getHeaps() {
		 if (heap == null) {
			 heap = new LinkedList<JavaHeap>();
			 heap.add(model.getHeap());
		 }
		 return heap;
	}

	@Override
	public List<JavaClassLoader> getJavaClassLoaders() {		
		return model.getJavaClassLoader();
	}

	@Override
	public ImagePointer getJavaVM() throws CorruptDataException {
		return new SimpleImagePointer(0);
	}

	@Override
	public JavaVMInitArgs getJavaVMInitArgs() throws DataUnavailable,
			CorruptDataException {
		throw new DataUnavailable();
	}

	@Override
	public List<JavaMonitor> getMonitors() {		
		return model.getJavaMonitors();
	}

	@Override
	public JavaObject getObjectAtAddress(ImagePointer pointer)
			throws CorruptDataException, IllegalArgumentException,
			MemoryAccessException, DataUnavailable {
		if(pointer==null) {
			CLogger.logr.log(Level.FINEST,"null pointer passed");
			throw new IllegalArgumentException("pointer is null");
		}
		CLogger.logr.log(Level.FINEST,"Ask for object at"+pointer.getAddress());
		
		JavaObject obj =  model.getObjectAtAddress(pointer.getAddress());
		
		if (obj == null) {
			throw new IllegalArgumentException("Unable to retrieve an object at 0x"+
					Long.toHexString(pointer.getAddress()));
		}
		
		return obj;
	}

	@Override
	public List<JavaThread> getThreads() {		
		return model.getThreads();
	}

	@Override
	public Object getTraceBuffer(String arg0, boolean arg1)
			throws CorruptDataException {
		return null;
	}

	@Override
	public String getFullVersion() throws CorruptDataException {
		return CJVMTIBinReader.CJVMTI_VERSION_STRING;
	}

	@Override
	public String getVersion() throws CorruptDataException {
		return CJVMTIBinReader.CJVMTI_VERSION_STRING;
	}

	@Override
	public File getSource() {
		return this.source;
	}

}
