/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/

package org.apache.kato.jvmti.javaruntime.model;

import javax.tools.diagnostics.image.CorruptDataException;
import javax.tools.diagnostics.image.DataUnavailable;
import javax.tools.diagnostics.image.ImageAddressSpace;
import javax.tools.diagnostics.image.ImagePointer;
import javax.tools.diagnostics.image.MemoryAccessException;

public final class SimpleImagePointer implements ImagePointer {
	private long address=0;
	public SimpleImagePointer(long id) {
		this.address=id;
	}

	@Override
	public ImagePointer add(long offset) {
		return new SimpleImagePointer(offset + address);
	}

	@Override
	public long getAddress() {		
		return address;
	}

	@Override
	public ImageAddressSpace getAddressSpace() {
		return null;
	}

	@Override
	public byte getByteAt(long index) throws MemoryAccessException,
			CorruptDataException {
		throw new MemoryAccessException(this, "No memory is available in this dump type.");
	}

	@Override
	public double getDoubleAt(long index)
			throws MemoryAccessException, CorruptDataException {
		throw new MemoryAccessException(this, "No memory is available in this dump type.");
	}

	@Override
	public float getFloatAt(long index)
			throws MemoryAccessException, CorruptDataException {
		throw new MemoryAccessException(this, "No memory is available in this dump type.");
	}

	@Override
	public int getIntAt(long index) throws MemoryAccessException,
			CorruptDataException {
		throw new MemoryAccessException(this, "No memory is available in this dump type.");
	}

	@Override
	public long getLongAt(long index) throws MemoryAccessException,
			CorruptDataException {
		throw new MemoryAccessException(this, "No memory is available in this dump type.");
	}

	@Override
	public ImagePointer getPointerAt(long index)
			throws MemoryAccessException, CorruptDataException {
		throw new MemoryAccessException(this, "No memory is available in this dump type.");
	}

	@Override
	public short getShortAt(long index)
			throws MemoryAccessException, CorruptDataException {
		throw new MemoryAccessException(this, "No memory is available in this dump type.");
	}

	@Override
	public boolean isExecutable() throws DataUnavailable {
		throw new DataUnavailable("No memory is available in this dump type.");
	}

	@Override
	public boolean isReadOnly() throws DataUnavailable {
		throw new DataUnavailable("No memory is available in this dump type.");
	}

	@Override
	public boolean isShared() throws DataUnavailable {
		throw new DataUnavailable("No memory is available in this dump type.");
	}
}