/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.common;

import java.io.IOException;
import java.util.List;



public abstract class DataProviderBackedArrayProvider  implements IArrayEntryProvider{

	private IDataProvider provider=null;
	
	public DataProviderBackedArrayProvider(IDataProvider provider) {
		this.provider=provider;
	}
	
	@Override
	public long getCurrentLocation() {

		try {
			return provider.getCurrentLocation();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return -1;
	}

	@Override
	public boolean moveRelativeElement(int seekNo) {

		while (seekNo > 0) {
			Object o = getCurrentElement();
			if (o == null)
				return false;
			seekNo--;
		}
		return true;
	}

	@Override
	public boolean moveToLocation(long l) {

		try {
		 	provider.moveTo(l);
		 	return true;
		} catch(IOException e) {
			return false;
		}
	}
	@Override
	public void getState(List state) {
		
		state.add("DataProviderBackedarrayProvider");
		provider.addState(state);
		
		
	}

	public IDataProvider getProvider() {
		return provider;
	}
}
