/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/

package org.apache.kato.common;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

/**
 * Map like class that can return the values as a {@link java.util.List}
 * This class has limited {@link java.util.HashMap} semantics in that objects
 * are retained in the map according to HashMap rules.  Value objects can be retrieved
 * from the class  by calling the {@link #values()} method.   Objects returned in the
 * list are in add order.    Updating a key via  {@link #put(Object, Object)} will result 
 * in the value moving to the end of the list.  
 * 
 * @experimental This is an experimental addition and may be removed at a later date 
 *
 */
public class ObjectMapList<K,V> {
	HashMap<K,V> map=new HashMap<K,V>();
	LinkedList<V> list=new LinkedList<V>();
	public List<V> values() {
		
		return list;
	}
	public V get(K key) {
		
		return map.get(key);
	}
	public void put(K key, V value) {
		if(map.containsKey(key)) {
			Object orig=map.get(key);
			list.remove(orig);
		}
		map.put(key, value);
		list.add(value);
	}
}