<?xml version="1.0"?>
<!--
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
-->
<xsl:stylesheet version="1.0" xmlns:html="http://www.w3.org/1999/xhtml"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format"
	xmlns:db="http://docbook.org/ns/docbook" xmlns:javadoc="http://apache.org/javadoc"
	xmlns="http://docbook.org/ns/docbook">

	<xsl:template match="/">

		<chapter version="5.0" xml:id="api-reference">
			<title>API Reference</title>
			<para>This chapter contains the details of the Java classes that
				comprise the API.</para>
			<xsl:for-each select="javadoc:javadoc/javadoc:package">
				<xsl:sort select="@name" />

				<sect1>

					<xsl:attribute name="xml:id">
    			<xsl:value-of select="@name" />
  			</xsl:attribute>
					<xsl:processing-instruction name="dbfo-need">
						height="4in"
					</xsl:processing-instruction>
					<title>
						<xsl:text>package </xsl:text>
						<xsl:value-of select="@name" />
					</title>
					<section>
						<xsl:copy-of select="javadoc:comment/*" />
					</section>
					<xsl:if test="count(./javadoc:interface)&gt;0">

						<table border="1" pgwide="0">
							<xsl:attribute name="xml:id">interfacelist.
    						<xsl:value-of select="@name" />
  						</xsl:attribute>
							<xsl:processing-instruction name="dbfo">
								keep-together="auto"
							</xsl:processing-instruction>
							<caption>Interface Summary</caption>
							<tgroup cols='3' align='left' colsep='1' rowsep='1'>
								<colspec colname='c1' colwidth="1.0*"/>
								<colspec colname='c2' colwidth="1.0*"/>
								<colspec colname='c3' colwidth="1.0*"/>
								<thead>
									<row>
										<xsl:processing-instruction name="dbfo">
											bgcolor="#CCCCFF"
										</xsl:processing-instruction>
										<entry>Name</entry>
										<entry namest="c2" nameend="c3">Summary</entry>
									</row>
								</thead>
								<tbody>
									<xsl:for-each select="./javadoc:interface">
										<row>
											<entry>
												<xsl:value-of select="@name" />
											</entry>
											<entry namest="c2" nameend="c3">
												<xsl:value-of select="javadoc:summary" />
											</entry>
										</row>
									</xsl:for-each>
								</tbody>
							</tgroup>
						</table>
					</xsl:if>

					<xsl:if test="count(./javadoc:class)&gt;0">
						<table border="1" pgwide="0">
							<xsl:attribute name="xml:id">classlist.
    						<xsl:value-of select="@name" />
    					</xsl:attribute>
							<xsl:processing-instruction name="dbfo">
								keep-together="auto"
							</xsl:processing-instruction>
							<caption>Class Summary</caption>
							<tgroup cols='3' align='left' colsep='1' rowsep='1'>
								<colspec colname='c1' colwidth="1.0*"/>
								<colspec colname='c2' colwidth="1.0*"/>
								<colspec colname='c3' colwidth="1.0*"/>
								<thead>
									<row>
										<xsl:processing-instruction name="dbfo">
											bgcolor="#CCCCFF"
										</xsl:processing-instruction>
										<entry>Name</entry>
										<entry namest="c2" nameend="c3">Summary</entry>
									</row>
								</thead>
								<tbody>
									<xsl:for-each select="./javadoc:class">
										<row>
											<entry>
												<xsl:value-of select="@name" />
											</entry>
											<entry namest="c2" nameend="c3">
												<xsl:value-of select="javadoc:summary" />
											</entry>
										</row>
									</xsl:for-each>
								</tbody>
							</tgroup>
						</table>
					</xsl:if>



					<!--  list class details -->
					<xsl:if test="count(./javadoc:interface | ./javadoc:class)&gt;0">
						<sect2>
							<title>Details</title>
							<xsl:for-each select="./javadoc:class | ./javadoc:interface">

								<sect3>
									<xsl:processing-instruction name="dbfo-need">
										height="4in"
									</xsl:processing-instruction>
									<title>
										<xsl:value-of select="local-name()" />
										<xsl:text> </xsl:text>
										<xsl:value-of select="@name" />
									</title>
									<para>
										<emphasis>
											<xsl:value-of select="javadoc:modifiers" />
											<xsl:text> </xsl:text>
											<xsl:value-of select="@name" />
											<xsl:if test="javadoc:extends">
												<xsl:text> extends </xsl:text>
												<xsl:value-of select="javadoc:extends" />
											</xsl:if>
										</emphasis>
									</para>
									<para>
										<xsl:copy-of select="javadoc:comment/*" />
									</para>
									<xsl:if test="count(./javadoc:field)&gt;0">
										<xsl:for-each select="./javadoc:field">
											<para>Field</para>
										</xsl:for-each>
									</xsl:if>
									<xsl:if test="count(./javadoc:constructor)&gt;0">
										<table border="1" pgwide="0">
											<xsl:attribute name="xml:id">constructors.
    						<xsl:value-of select="@name" />
  						</xsl:attribute>
											<xsl:processing-instruction name="dbfo">
												keep-together="auto"
											</xsl:processing-instruction>
											<caption>
												<xsl:value-of select="@name" />
												Constructor Summary
											</caption>
											<thead>
												<tr>
													<xsl:processing-instruction
														name="dbfo">
														bgcolor="#CCCCFF"
													</xsl:processing-instruction>
													<td>Constructor</td>

												</tr>
											</thead>
											<tbody>
												<xsl:for-each select="./javadoc:constructor">
													<tr>
														<td>
														    <!--  signature  -->
															<emphasis>
																<xsl:value-of select="javadoc:modifiers" />
																<xsl:text> </xsl:text>
																<xsl:value-of select="../@name" />
																<!--  parameters -->
																	<xsl:text>(</xsl:text>
																	<xsl:if test="javadoc:parameters">
																		<xsl:for-each select="javadoc:parameters/javadoc:parameter">
																			<xsl:value-of select="@type" />
																			<xsl:text> </xsl:text>
																			<xsl:value-of select="@name" />
																		</xsl:for-each>
																	</xsl:if>
																	<xsl:text>)</xsl:text>
															</emphasis>
															<para>
																<xsl:copy-of select="javadoc:comment/*" />
															</para>
														</td>
													</tr>
												</xsl:for-each>
											</tbody>
										</table>
									</xsl:if>

									<!-- List Methods -->

									<xsl:if test="count(./javadoc:method)&gt;0">
										<table border="1" pgwide="0">
											<xsl:processing-instruction name="dbfo">
												keep-together="auto"
											</xsl:processing-instruction>
											<caption>
												<xsl:value-of select="@name" />
												Methods
											</caption>
											<tgroup cols='10' align='left' colsep='1' rowsep='1'>
												<colspec colname='a' colwidth="1.0*"/>
												<colspec colname='b' colwidth="1.0*"/>
												<colspec colname='c' colwidth="1.0*"/>
												<colspec colname='d' colwidth="1.0*"/>
												<colspec colname='e' colwidth="1.0*"/>
												<colspec colname='f' colwidth="1.0*"/>
												<colspec colname='g' colwidth="1.0*"/>
												<colspec colname='h' colwidth="1.0*"/>
												<colspec colname='i' colwidth="1.0*" />
												<colspec colname='j' colwidth="1.0*"/>
												<thead>
													<row>
														<xsl:processing-instruction
															name="dbfo">
															bgcolor="#CCCCFF"
														</xsl:processing-instruction>
														<entry namest="a" nameend="j">Methods</entry>
													</row>
												</thead>
												<tbody>

													<xsl:for-each select="./javadoc:method">
														<row>
															<entry namest="a" nameend="c">
																<command>
																	<xsl:value-of select="@name" />
																</command>
															</entry>

															<!--
																write out details of method. Signature Comments
																Parameters Returns Throws
															-->

															<entry namest="d" nameend="j">
																<emphasis>
																	<!--  signature -->
																	<xsl:value-of select="javadoc:modifiers" />
																	<xsl:text> </xsl:text>
																	<!-- return type -->
																	<xsl:value-of select="javadoc:returntype/javadoc:name" />
																	<xsl:text> </xsl:text>
																	<!-- name -->
																	<xsl:value-of select="@name" />
																	<!--  parameters -->
																	<xsl:text>(</xsl:text>
																	<xsl:if test="javadoc:parameters">
																		<xsl:for-each select="javadoc:parameters/javadoc:parameter">
																			<xsl:value-of select="@type" />
																			<xsl:text> </xsl:text>
																			<xsl:value-of select="@name" />
																		</xsl:for-each>
																	</xsl:if>
																	<xsl:text>)</xsl:text>
																</emphasis>
																<!--  comments -->
																<para>
																	<xsl:copy-of select="javadoc:comment/*" />
																</para>

																<!-- returns -->
																<xsl:if test="count(./javadoc:returns/javadoc:return)&gt;0">
																	<section>
																		<informaltable frame="none">
																			<tgroup cols='1' align='left' colsep='0'
																				rowsep='0'>
																				<colspec name="a" colwidth="1.0*"/>
																				<thead>
																					<row>
																						<entry>Returns</entry>
																					</row>
																				</thead>
																				<tbody>

																					<xsl:for-each select="./javadoc:returns/javadoc:return">
																						<row>
																							<entry>
																								<xsl:value-of select="." />
																							</entry>
																						</row>
																					</xsl:for-each>
																				</tbody>
																			</tgroup>
																		</informaltable>
																	</section>
																</xsl:if>
																<!-- parameters -->
																<xsl:if
																	test="count(./javadoc:paramdescs/javadoc:paramdesc)&gt;0">
																	<section>
																		<informaltable>
																			<tgroup cols='5' align='left' colsep='0'
																				rowsep='0'>
																				<colspec name="a" colwidth="1.0*"/>
																				<colspec name="b" colwidth="1.0*"/>
																				<colspec name="c" colwidth="1.0*"/>
																				<colspec name="d" colwidth="1.0*"/>
																				<colspec name="e" colwidth="1.0*"/>
																				<thead>
																					<row>
																						<entry namest="a" nameend="e">Parameters
																						</entry>
																					</row>
																				</thead>
																				<tbody>

																					<xsl:for-each
																						select="./javadoc:paramdescs/javadoc:paramdesc">
																						<row>
																							<entry>
																								<xsl:value-of select="@name" />
																							</entry>
																							<entry namest="b" nameend="e">
																								<xsl:value-of select="." />
																							</entry>
																						</row>
																					</xsl:for-each>
																				</tbody>
																			</tgroup>
																		</informaltable>
																	</section>
																</xsl:if>
																<!-- throqs -->
																<xsl:if test="count(./javadoc:throws/javadoc:throw)&gt;0">
																	<section>
																		<informaltable frame="none">
																			<tgroup cols='5' align='left' colsep='0'
																				rowsep='0'>
																				<colspec name="a" colwidth="1.0*"/>
																				<colspec name="b" colwidth="1.0*"/>
																				<colspec name="c" colwidth="1.0*"/>
																				<colspec name="d" colwidth="1.0*"/>
																				<colspec name="e" colwidth="1.0*"/>
																				<thead>
																					<row>
																						<entry namest="a" nameend="e">Throws
																						</entry>
																					</row>
																				</thead>
																				<tbody>

																					<xsl:for-each select="./javadoc:throws/javadoc:throw">
																						<row>
																							<entry namest="a" nameend="b">
																								<xsl:value-of select="@name" />
																							</entry>
																							<entry namest="c" nameend="e">
																								<xsl:value-of select="." />
																							</entry>
																						</row>
																					</xsl:for-each>
																				</tbody>
																			</tgroup>
																		</informaltable>
																	</section>
																</xsl:if>
															</entry>
														</row>
													</xsl:for-each>
												</tbody>
											</tgroup>
										</table>
									</xsl:if>
								</sect3>

							</xsl:for-each>
						</sect2>
					</xsl:if>
				</sect1>
			</xsl:for-each>
		</chapter>
	</xsl:template>
</xsl:stylesheet>