/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.hprof.image;

import javax.tools.diagnostics.image.CorruptData;
import javax.tools.diagnostics.image.ImagePointer;

/**
 * Our implementation of the CorruptData interface.
 * 
 */
public class CorruptDataImpl implements CorruptData {
	private ImagePointer address;
	private String message;
	
	public CorruptDataImpl(ImagePointer address, String message) {
		this.address = address;
		this.message = message;
	}
	 
	public CorruptDataImpl(String message) {
		this.message = message;
	}
	
	@Override
	public ImagePointer getAddress() {
		return address;
	}

	/**
	 * Return string representing this corrupt data like so:
	 * 	"Corrupt Data: [message]"
	 * or
	 *  "Corrupt Data: @ 0x[address] [message]".
	 */
	@Override
	public String toString() {
		if (address == null) {
			return "CorruptData: "+message;
		} else {
			return "CorruptData: @ 0x" +
				Long.toHexString(address.getAddress())+" "+message;
		}
	}
}
