/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.hprof.image;

import javax.tools.diagnostics.image.CorruptDataException;
import javax.tools.diagnostics.image.DataUnavailable;
import javax.tools.diagnostics.image.ImageAddressSpace;
import javax.tools.diagnostics.image.ImagePointer;
import javax.tools.diagnostics.image.MemoryAccessException;

public class ImagePointerImpl implements ImagePointer {
	private long address=0;
	private ImageAddressSpace addressSpace;
	
	public ImagePointerImpl(long address, ImageAddressSpace addressSpace) {
		this.address=address;
		this.addressSpace = addressSpace;
	}
	@Override
	public ImagePointer add(long offset) {		
		return new ImagePointerImpl(address+offset, addressSpace);
	}
 
	@Override
	public long getAddress() {
		
		return address;
	}

	@Override
	public ImageAddressSpace getAddressSpace() {		
		return addressSpace;
	}

	@Override
	public byte getByteAt(long index) throws MemoryAccessException,
			CorruptDataException {
		
		throwError();
		return 0;
	}

	@Override
	public double getDoubleAt(long index) throws MemoryAccessException,
			CorruptDataException {
		throwError();
		return 0;
	}

	@Override
	public float getFloatAt(long index) throws MemoryAccessException,
			CorruptDataException {
		throwError();
		return 0;
	}

	@Override
	public int getIntAt(long index) throws MemoryAccessException,
			CorruptDataException {
		throwError();
		return 0;
	}

	@Override
	public long getLongAt(long index) throws MemoryAccessException,
			CorruptDataException {
		throwError();
		return 0;
	}

	@Override
	public ImagePointer getPointerAt(long index) throws MemoryAccessException,
			CorruptDataException {
		throwError();
		return null;
	}

	@Override
	public short getShortAt(long index) throws MemoryAccessException,
			CorruptDataException {
		throwError();
		return 0;
	}

	@Override
	public boolean isExecutable() throws DataUnavailable {
		throw new DataUnavailable();
		
	}

	@Override
	public boolean isReadOnly() throws DataUnavailable {
		throw new DataUnavailable();
	}

	@Override
	public boolean isShared() throws DataUnavailable {
		throw new DataUnavailable();
	}
	
	private void throwError() throws MemoryAccessException {
		throw new MemoryAccessException(this,"memory access invalid with hprof dump");
	}

	/**
	 * Is this a good hashcode?
	 */
	@Override
	public int hashCode() {
		return (int) ( address % Integer.MAX_VALUE );
	}
	
	/**
	 * This is the same ImagePointer if the addresses are the same in the same addresspace.
	 */
	@Override
	public boolean equals(Object obj) {
		if ( obj == null){
			return false;
		} else if (!(obj instanceof ImagePointerImpl)) {
			return false;
		} 
		
		ImageAddressSpace otherAddressSpace = ((ImagePointerImpl) obj).getAddressSpace();
		
		if (otherAddressSpace.equals(getAddressSpace())) {
			return address == ((ImagePointerImpl)obj).getAddress();
		}
		
		return false;		
	}
}
