/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.hprof.java;



import junit.framework.TestCase;

public class JavaObjectInstanceImplTest extends TestCase {
	JavaObjectInstanceImpl instance;
	IJavaHeapInternal heap;
	GCInstanceHeapDumpRecordTest gcInstance;
	
	public void setUp() {
		heap = new JavaHeapImplTest();
		gcInstance =  new GCInstanceHeapDumpRecordTest();
		gcInstance.setID(1);
		instance = new JavaObjectInstanceImpl(heap, gcInstance);
	}
	
	public void testGetHeap() throws Exception {
		assertEquals("JavaHeaps don't match.", heap, instance.getHeap());
	}
	
	public void testGetID() throws Exception {
		assertEquals("ID incorrectly retrieved", gcInstance.getID(), instance.getID().getAddress());
	}
	
	public void testGetObjectID() throws Exception {
		assertEquals("ID incorrectly retrieved", gcInstance.getID(), instance.getObjectID());
	}
	public void testEqualsNull() throws Exception {
		assertFalse("equals(null) should be false.",instance.equals(null));
	}
	
	public void testEqualsWrongType() throws Exception {
		assertFalse("equals(String) should be false.", instance.equals("Completely wrong type"));
	}
	
	public void testEquals() throws Exception {
		assertTrue("instance.equals(instance) should be true.", instance.equals(instance));
	}
	
	public void testIsArray() throws Exception {
		assertFalse("instance.isArray() should be false", instance.isArray());
	}
	
	public void testGetArraySize() throws Exception {
		try{
			int size = instance.getArraySize();
			fail("instance.getArraySize() didn't throw IllegalArgumentException - returned "+size);
		} catch(IllegalArgumentException e) {
			// Normal case.
		}
	}
}
