/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/


#include "cjvmti.h"
#include "addObject.h"
#include "addClass.h"

typedef union{
	jbyte * bArray;
	jchar * cArray;
	jdouble * dArray;
	jfloat * fArray;
	jint * iArray;
	jlong * jArray;
	jshort * sArray;
	jboolean * zArray;
} primArrays;

typedef union{
	jbyte b;
	jchar c;
	jdouble d;
	jfloat f;
	jint i;
	jlong j;
	jshort s;
	jboolean z;
} primitives;

// Saves a field to the dump file returning the position in the file it was saved to.
// type indicates where the field value came from instance, local or static.
FPOS_T addFieldValue(int type, jobject obj, jfieldID field, char * signature,
		jthread thread, jint depth, jint slot, int referenceDepth) {

	// Primitives
	
	primitives prim;

	// Arrays
	
	primArrays primArr;

	int szArray;
	int tempI;
	jobject arrayElement;
	jobject l = 0;
	jboolean isCopy;
	jvmtiError err;
	int counter;
	FPOS_T position = -1;
	FPOS_T * arrayFields;
	struct JClass * arrayType;
		tempI = 0;
		memset(&primArr, 0, sizeof(primArr));
		memset(&prim, 0, sizeof(prim));

	position = FTELL(variableFile);
	switch (*signature) {
	case 'B':
		switch (type) {
		case CJVMTI_INSTANCE_VAR:
			prim.b = (*jniptr)->GetByteField(jniptr, obj, field);
			break;
		case CJVMTI_LOCAL_VAR:
			err = (*env)->GetLocalInt(env, thread, depth, slot, &tempI);
			prim.b = (jbyte) tempI;
			break;
		case CJVMTI_STATIC_VAR:
			prim.b = (*jniptr)->GetStaticByteField(jniptr, obj, field);
			break;
		}
		writeIDNoSize(CJVMTI_BYTE, variableFile);
		fwrite(&prim.b, sizeof(jbyte), 1, variableFile);
		break;
	case 'C':
		switch (type) {
		case CJVMTI_INSTANCE_VAR:
			prim.c = (*jniptr)->GetCharField(jniptr, obj, field);
			break;
		case CJVMTI_LOCAL_VAR:
			err = (*env)->GetLocalInt(env, thread, depth, slot, &tempI);
			prim.c = (jchar) tempI;
			break;
		case CJVMTI_STATIC_VAR:
			prim.c = (*jniptr)->GetStaticCharField(jniptr, obj, field);
			break;
		}
		writeIDNoSize(CJVMTI_CHAR, variableFile);
		fwrite(&prim.c, sizeof(jchar), 1, variableFile);
		break;
	case 'D':
		switch (type) {
		case CJVMTI_INSTANCE_VAR:
			prim.d = (*jniptr)->GetDoubleField(jniptr, obj, field);
			break;
		case CJVMTI_LOCAL_VAR:
			err = (*env)->GetLocalDouble(env, thread, depth, slot, &prim.d);
			break;
		case CJVMTI_STATIC_VAR:
			prim.d = (*jniptr)->GetStaticDoubleField(jniptr, obj, field);
			break;
		}
		writeIDNoSize(CJVMTI_DOUBLE, variableFile);
		fwrite(&prim.d, sizeof(jdouble), 1, variableFile);
		break;
	case 'F':
		switch (type) {
		case CJVMTI_INSTANCE_VAR:
			prim.f = (*jniptr)->GetFloatField(jniptr, obj, field);
			break;
		case CJVMTI_LOCAL_VAR:
			err = (*env)->GetLocalFloat(env, thread, depth, slot, &prim.f);
			break;
		case CJVMTI_STATIC_VAR:
			prim.f = (*jniptr)->GetStaticFloatField(jniptr, obj, field);
			break;
		}
		writeIDNoSize(CJVMTI_FLOAT, variableFile);
		fwrite(&prim.f, sizeof(jfloat), 1, variableFile);

		break;
	case 'I':
		switch (type) {
		case CJVMTI_INSTANCE_VAR:
			prim.i = (*jniptr)->GetIntField(jniptr, obj, field);
			break;
		case CJVMTI_LOCAL_VAR:
			err = (*env)->GetLocalInt(env, thread, depth, slot, &prim.i);
			break;
		case CJVMTI_STATIC_VAR:
			prim.i = (*jniptr)->GetStaticIntField(jniptr, obj, field);
			break;
		}
		writeIDNoSize(CJVMTI_INT, variableFile);
		fwrite(&prim.i, sizeof(jint), 1, variableFile);
		break;
	case 'J':
		switch (type) {
		case CJVMTI_INSTANCE_VAR:
			prim.j = (*jniptr)->GetLongField(jniptr, obj, field);
			break;
		case CJVMTI_LOCAL_VAR:
			err = (*env)->GetLocalLong(env, thread, depth, slot, &prim.j);
			break;
		case CJVMTI_STATIC_VAR:
			prim.j = (*jniptr)->GetStaticLongField(jniptr, obj, field);
			break;
		}
		writeIDNoSize(CJVMTI_LONG, variableFile);
		fwrite(&prim.j, sizeof(jlong), 1, variableFile);
		break;
	case 'L':
		if (maxReferenceDepth != 0 && maxReferenceDepth <= referenceDepth){
			writeIDNoSize(CJVMTI_NULL_OBJECT, variableFile);
			break;
		}
		switch (type) {
		case CJVMTI_INSTANCE_VAR:
			l = (*jniptr)->GetObjectField(jniptr, obj, field);
			break;
		case CJVMTI_LOCAL_VAR:
			err = (*env)->GetLocalObject(env, thread, depth, slot, &l);
			if (err != JVMTI_ERROR_NONE) {
				printf("Error getting local object %d\n", err);
			}
			break;
		case CJVMTI_STATIC_VAR:
			l = (*jniptr)->GetStaticObjectField(jniptr, obj, field);
			break;
		case CJVMTI_ARRAY_VAR:
			l = obj;
		}

		if (l != 0) {
			position = addObject(l, referenceDepth+1)->positionInFile;
		} else {
			writeIDNoSize(CJVMTI_NULL_OBJECT, variableFile);
		}
		break;
	case 'S':
		switch (type) {
		case CJVMTI_INSTANCE_VAR:
			prim.s = (*jniptr)->GetShortField(jniptr, obj, field);
			break;
		case CJVMTI_LOCAL_VAR:
			err = (*env)->GetLocalInt(env, thread, depth, slot, &tempI);
			prim.s = (jshort) tempI;
			break;
		case CJVMTI_STATIC_VAR:
			prim.s = (*jniptr)->GetStaticShortField(jniptr, obj, field);
			break;
		}
		writeIDNoSize(CJVMTI_SHORT, variableFile);
		fwrite(&prim.s, sizeof(jshort), 1, variableFile);
		break;
	case 'Z':
		switch (type) {
		case CJVMTI_INSTANCE_VAR:
			prim.z = (*jniptr)->GetBooleanField(jniptr, obj, field);
			break;
		case CJVMTI_LOCAL_VAR:
			err = (*env)->GetLocalInt(env, thread, depth, slot, &tempI);
			prim.z = (jboolean) tempI;
			break;
		case CJVMTI_STATIC_VAR:
			prim.z = (*jniptr)->GetStaticBooleanField(jniptr, obj, field);
			break;
		}
		writeIDNoSize(CJVMTI_BOOLEAN, variableFile);
		fwrite(&prim.z, sizeof(jboolean), 1, variableFile);

		break;
	case '[':
		signature++;
		if ((*signature == 'L') || (*signature == '[')) {
			if (maxReferenceDepth != 0 && maxReferenceDepth <= referenceDepth){
				writeIDNoSize(CJVMTI_NULL_OBJECT, variableFile);
				break;
			}
			switch (type) {
			case CJVMTI_INSTANCE_VAR:
				l = (*jniptr)->GetObjectField(jniptr, obj, field);
				break;
			case CJVMTI_LOCAL_VAR:
				err = (*env)->GetLocalObject(env, thread, depth, slot, &l);
				break;
			case CJVMTI_STATIC_VAR:
				l = (*jniptr)->GetStaticObjectField(jniptr, obj, field);
				break;
			case CJVMTI_ARRAY_VAR:
				l = obj;
				break;
			}
			if (l == 0) {
				writeIDNoSize(CJVMTI_NULL_OBJECT, variableFile);
			} else {
				FSEEK(variableFile, 0, SEEK_END);
				arrayType = addClassDetails(
						(*jniptr)->GetObjectClass(jniptr, l));
				szArray = (*jniptr)->GetArrayLength(jniptr, l);
				arrayFields = NULL;
				if (szArray > 0) {
					// Save field references in memory
					arrayFields = malloc(sizeof(FPOS_T) * szArray);
					for (counter = 0; counter < szArray; counter++) {
						FSEEK(variableFile, 0, SEEK_END);
						arrayElement = (*jniptr)->GetObjectArrayElement(jniptr,
								l, counter);
						if (arrayElement == 0) {
							arrayFields[counter] = CJVMTI_NULL_OBJECT;
						} else {
							arrayFields[counter] = addFieldValue(
									CJVMTI_ARRAY_VAR, arrayElement, 0,
									(arrayType->name) + 1, 0, 0, 0, referenceDepth+1);
							if (arrayFields[counter] == 0){
								printf("Found left over ref\n");
								(*jniptr)->DeleteLocalRef(jniptr, arrayElement);
							}
						}
					}
				}
				FSEEK(variableFile, 0, SEEK_END);
				position = FTELL(variableFile);

				// Write out array references / array 
				writeIDNoSize(CJVMTI_OBJECT_ARRAY, variableFile);
				writeReference(arrayType->positionInFile, variableFile);
				writeJint(szArray, variableFile);
				if (szArray > 0) {
					for (counter = 0; counter < szArray; counter++) {
						if (arrayFields[counter] == 0) {
							writeReference(CJVMTI_NULL_OBJECT, variableFile);
						} else {
							writeReference(arrayFields[counter], variableFile);
						}
					}
				}
				free(arrayFields);
			}
			break;
		} else {
			switch (type) {
			case CJVMTI_INSTANCE_VAR:
				l = (*jniptr)->GetObjectField(jniptr, obj, field);
				break;
			case CJVMTI_LOCAL_VAR:
				err = (*env)->GetLocalObject(env, thread, depth, slot, &l);
				break;
			case CJVMTI_STATIC_VAR:
				l = (*jniptr)->GetStaticObjectField(jniptr, obj, field);
				break;
			case CJVMTI_ARRAY_VAR:
				l = obj;
				break;
			}
			// TODO correct this, do we have a null object before we declare the class type, or use the signature..
			if (l == 0) {
				writeIDNoSize(CJVMTI_NULL_OBJECT, variableFile);
				break;
			}
			szArray = (*jniptr)->GetArrayLength(jniptr, l);

			switch (*signature) {
			//	B 	byte 	signed byte
			//	C 	char 	Unicode character
			//	D 	double 	double-precision floating-point value
			//	F 	float 	single-precision floating-point value
			//	I 	int 	integer
			//	J 	long 	long integer
			//	S 	short 	signed short
			//	Z 	boolean 	true or false
			case 'B':
				if (l == 0) {
					writeIDNoSize(CJVMTI_NULL_OBJECT, variableFile);
				} else {
					writeIDNoSize(CJVMTI_BYTE_ARRAY, variableFile);
					writeJint(szArray, variableFile);
					primArr.bArray
							= (*jniptr)->GetByteArrayElements(jniptr, l,
									&isCopy);

					for (counter = 0; counter < szArray; counter++) {
						fwrite(&primArr.bArray[counter], sizeof(jbyte), 1, variableFile);
					}
					(*jniptr)->ReleaseByteArrayElements(jniptr, l, primArr.bArray,JNI_ABORT);
				}
				break;
			case 'C':
				if (l == 0) {
					writeIDNoSize(CJVMTI_NULL_OBJECT, variableFile);
				} else {
					writeIDNoSize(CJVMTI_CHAR_ARRAY, variableFile);
					writeJint(szArray, variableFile);
					primArr.cArray
							= (*jniptr)->GetCharArrayElements(jniptr, l,
									&isCopy);
					for (counter = 0; counter < szArray; counter++) {
						fwrite(&primArr.cArray[counter], sizeof(jchar), 1, variableFile);
					}
					(*jniptr)->ReleaseCharArrayElements(jniptr, l, primArr.cArray,JNI_ABORT);
				}
				break;
			case 'D':
				if (l == 0) {
					writeIDNoSize(CJVMTI_NULL_OBJECT, variableFile);
				} else {
					writeIDNoSize(CJVMTI_DOUBLE_ARRAY, variableFile);
					writeJint(szArray, variableFile);

					primArr.dArray = (*jniptr)->GetDoubleArrayElements(jniptr, l,
							&isCopy);
					for (counter = 0; counter < szArray; counter++) {
						fwrite(&primArr.dArray[counter], sizeof(jdouble), 1,
								variableFile);
					}
					(*jniptr)->ReleaseDoubleArrayElements(jniptr, l, primArr.dArray,JNI_ABORT);
				}
				break;
			case 'F':
				if (l == 0) {
					writeIDNoSize(CJVMTI_NULL_OBJECT, variableFile);
				} else {
					writeIDNoSize(CJVMTI_FLOAT_ARRAY, variableFile);
					writeJint(szArray, variableFile);
					primArr.fArray = (*jniptr)->GetFloatArrayElements(jniptr, l,
							&isCopy);
					for (counter = 0; counter < szArray; counter++) {
						fwrite(&primArr.fArray[counter], sizeof(jfloat), 1,
								variableFile);
					}
					(*jniptr)->ReleaseFloatArrayElements(jniptr, l, primArr.fArray,JNI_ABORT);
				}
				break;
			case 'I':
				if (l == 0) {
					writeIDNoSize(CJVMTI_NULL_OBJECT, variableFile);
				} else {
					writeIDNoSize(CJVMTI_INT_ARRAY, variableFile);
					writeJint(szArray, variableFile);
					primArr.iArray = (*jniptr)->GetIntArrayElements(jniptr, l, &isCopy);
					for (counter = 0; counter < szArray; counter++) {
						fwrite(&primArr.iArray[counter], sizeof(jint), 1, variableFile);
					}
					(*jniptr)->ReleaseIntArrayElements(jniptr, l, primArr.iArray,JNI_ABORT);
				}
				break;

			case 'J':
				if (l == 0) {
					writeIDNoSize(CJVMTI_NULL_OBJECT, variableFile);
				} else {
					writeIDNoSize(CJVMTI_LONG_ARRAY, variableFile);
					writeJint(szArray, variableFile);
					primArr.jArray
							= (*jniptr)->GetLongArrayElements(jniptr, l,
									&isCopy);
					for (counter = 0; counter < szArray; counter++) {
						fwrite(&primArr.jArray[counter], sizeof(jlong), 1, variableFile);
					}
					(*jniptr)->ReleaseLongArrayElements(jniptr, l, primArr.jArray,JNI_ABORT);
				}
				break;
			case 'S':
				if (l == 0) {
					writeIDNoSize(CJVMTI_NULL_OBJECT, variableFile);
				} else {
					writeIDNoSize(CJVMTI_SHORT_ARRAY, variableFile);
					writeJint(szArray, variableFile);
					primArr.sArray = (*jniptr)->GetShortArrayElements(jniptr, l,
							&isCopy);
					for (counter = 0; counter < szArray; counter++) {
						fwrite(&primArr.sArray[counter], sizeof(jshort), 1,
								variableFile);
					}
					(*jniptr)->ReleaseShortArrayElements(jniptr, l, primArr.sArray,JNI_ABORT);
				}
				break;
			case 'Z':
				if (l == 0) {
					writeIDNoSize(CJVMTI_NULL_OBJECT, variableFile);
				} else {
					writeIDNoSize(CJVMTI_BOOLEAN_ARRAY, variableFile);
					writeJint(szArray, variableFile);
					primArr.zArray = (*jniptr)->GetBooleanArrayElements(jniptr, l,
							&isCopy);
					for (counter = 0; counter < szArray; counter++) {
						fwrite(&primArr.zArray[counter], sizeof(jboolean), 1,
								variableFile);
					}
					(*jniptr)->ReleaseBooleanArrayElements(jniptr, l, primArr.zArray,JNI_ABORT);
				}
				break;
			}
		}
	}
	/*free(bArray);
	free(cArray);
	free(dArray);
	free(fArray);
	free(iArray);
	free(jArray);
	free(sArray); 
	free(zArray); */
	return position;
}
