/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/

#ifndef CJVMTI_H_
#define CJVMTI_H_

// Cross platform file operations all based on __int64
#ifdef __linux__
#define _FILE_OFFSET_BITS 64
#elif WIN32
// Uncomment to detect memory leaks with visual c++
//#define __DEBUG_CJVMTI__
#endif

#ifdef __DEBUG_CJVMTI__
#define _CRTDBG_MAP_ALLOC
	#include <stdlib.h>
	#include <crtdbg.h>
#include <windows.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <time.h>
#include <malloc.h>

#ifdef  __linux__
#define FPOS_T off_t
#define FSEEK(x,y,z) fseeko((x),(y),(z))
#define FTELL(x) ftello((x))
#define FGETPOS(x,y) (*(y))=ftello((x))
#define FSETPOS(x,y) fseeko((x), (*(y)), SEEK_SET)
#include <stdint.h>

typedef int64_t INT64;
#elif   __aix__
#define FPOS_T off_t
#define FSEEK(x,y,z) fseeko((x),(y),(z))
#define FTELL(x) ftello((x))
#define FGETPOS(x,y) (*(y))=ftello((x))
#define FSETPOS(x,y) fseeko((x), (*(y)), SEEK_SET)
#elif   WIN32
#define FPOS_T __int64
#define FSEEK(x,y,z) _fseeki64((x),(y),(z))
#define FTELL(x) _ftelli64((x))
#define FGETPOS(x,y) (*(y))=_ftelli64((x))
#define FSETPOS(x,y) _fseeki64((x), (*(y)), SEEK_SET)

/* Prototypes for 64 bit functions */
extern __int64 _ftelli64(FILE *stream);
extern int _fseeki64(FILE *stream, __int64 offset, int origin);
typedef _int64 INT64;
#else
#error "unable to compile on this platform"
#endif

#include "queue.h"
#include "jvmti.h"

struct JObject {
	int state; // 0 not prepared 1 prepared 2 is references filled
	FPOS_T positionInFile; // Actual object position in dump file
	FPOS_T referenceLocation; // Used to store where references should be saved
	int totalFieldCount; // Number of fields after following the superclass hierarchy
    enum { OBJECT, CLASS, CLASSLOADER} type; // Identifies what type of object this is
	char depth; // Used to limit depth references are followed to
};

struct JClass {
	struct JObject obj; // Class object
	int filledIn; // Check class has been correctly initialised
	char * name; // Class name
	FPOS_T positionInFile; // Actual position in dump file
	FPOS_T classLoader; // Class loader position
	FPOS_T staticFieldStart; // Where static fields should be saved
	struct JClass * superClass; 
	int numStaticFields;
	int numInstanceFields;
	jfieldID * instanceFields; 
	char * * instanceFieldSignatures; // each ref is jvmti, malloced normally
	char * * instanceFieldGenSig; // each ref is jvmti, malloced normally
	jint * instanceModifiers; // malloced normally
};

struct JClassLoader {
	struct JObject obj; // Classloader object
    FPOS_T positionInFile; // Position of classloader in dump file
    jobject objID;    // object ID
};

#define CJVMTI_LOCAL_VAR  0
#define CJVMTI_STATIC_VAR 1
#define CJVMTI_INSTANCE_VAR 2
#define CJVMTI_ARRAY_VAR 3
// Field values
#define CJVMTI_BYTE 0x01
#define CJVMTI_CHAR 0x02
#define CJVMTI_DOUBLE 0x03
#define CJVMTI_FLOAT 0x04
#define CJVMTI_INT 0x05
#define CJVMTI_LONG 0x06
#define CJVMTI_OBJECT 0x07
#define CJVMTI_SHORT 0x08
#define CJVMTI_BOOLEAN 0x09
// Field arrays
#define CJVMTI_OBJECTARRAY 0x0a
#define CJVMTI_BYTE_ARRAY 0x0b
#define CJVMTI_CHAR_ARRAY 0x0c
#define CJVMTI_DOUBLE_ARRAY 0x0d
#define CJVMTI_FLOAT_ARRAY 0x0e
#define CJVMTI_INT_ARRAY 0x0f
#define CJVMTI_LONG_ARRAY 0x10
#define CJVMTI_OBJECT_ARRAY 0x11
#define CJVMTI_SHORT_ARRAY 0x12
#define CJVMTI_BOOLEAN_ARRAY 0x13
// Structure ID's
#define CJVMTI_METHOD 0x14
#define CJVMTI_THREAD_GROUP 0x15
#define CJVMTI_THREAD 0x16
#define CJVMTI_FRAME 0x17
#define CJVMTI_FIELD_MODIFIERS 0x18
#define CJVMTI_MONITORS 0x19
#define CJVMTI_CLASSLOADER 0x1a
#define CJVMTI_CLASSLOADERS 0x1b
// General error's and extra additions
#define CJVMTI_LOCAL_NATIVECALL 0x1c
#define CJVMTI_CLASS 0x1d
#define CJVMTI_CLASS_INSTANCE_FIELDS 0x1e
#define CJVMTI_CLASS_STATIC_FIELDS 0x1f
#define CJVMTI_CLASS_FIELDS 0x20
#define CJVMTI_LOCAL_VARIABLE 0x21
#define CJVMTI_JVMTI_ERROR 0x22
#define CJVMTI_SUPERCLASS_FIELDS 0x23
#define CJVMTI_CLASS_INTERFACES 0x24

#define CJVMTI_STRING 0x25; /* Not Actually used*/
#define CJVMTI_NULL_OBJECT 0x00
#define CJVMTI_NULL_OBJECT_ARRAY 0x00

/* Global variables */
jvmtiEnv *env;
JavaVM *vmptr;
JNIEnv *jniptr;
jrawMonitorID lock;
FILE *fp; /* log file */
FILE *variableFile; /* dump file */
int maxReferenceDepth;
int maxFrameDepth;
int printDepth;
struct queueADT * qObjects; /* References waiting to be followed in objects */
struct queueADT * qStaticFields; /* References waiting to be followed in classes */
struct queueADT * toDetag; /* Memory that needs to be freed/objects that need to be detagged */
struct queueADT * qClassLoaders; /* Memory that needs to be freed/objects that need to be detagged */
long objectCount;
long staticsCount;
struct JObject nullObject;
struct JClassLoader *systemLoader; /* System classloader has no object on heap to tag. Use this */
#ifdef __DEBUG_CJVMTI__
HANDLE hLogFile;
#endif
// End globals



static FPOS_T waitingToBeWritten = 0x7fffffffffffffffL; // Used to find unwritten references in output files, should not occur
int clearObjectQueue(); // Clear references to objects made by other objects so far.
void printfR(const char * _Format, ...);
void printfd(const char * _Format, ...);
int writeIDNoSize(char id, FILE * fp);
int writeString(char * tString, FILE * fp);
int writeJint(jint i, FILE * fp);
int writeReference(FPOS_T pt, FILE * fp);
int writeLong(jlong j, FILE *fp);
int writeBool(jboolean b, FILE * fp);
static void JNICALL dataDumpRequest(jvmtiEnv *jvmti);
#endif
