/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tck.harness;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintStream;
import java.util.LinkedList;
import java.util.List;

public class ProcessLauncher {

	private List standardEnvVars = new LinkedList();
	private String suffix = "";
	private TCKReporter reporter=null;
	
	public ProcessLauncher() {
		reporter=new TCKReporter();
	}
	public ProcessLauncher(TCKReporter reporter) {
		this.reporter=reporter;
	}
	public int launch(File workingDir, List envvars, List list)
			throws IOException {

		List myEnvars = new LinkedList();
		myEnvars.addAll(standardEnvVars);
		myEnvars.addAll(envvars);

		int size = list.size();
		String[] parms = (String[]) list.toArray(new String[size]);
		String[] vars = (String[]) myEnvars
				.toArray(new String[myEnvars.size()]);

		return launch(workingDir, vars, parms, null);
	}

	public int launch(File workingDir, String[] parms) throws IOException {
		List myEnvars = new LinkedList();
		myEnvars.addAll(standardEnvVars);
		String[] vars = (String[]) myEnvars
				.toArray(new String[myEnvars.size()]);
		return launch(workingDir, vars, parms, null);
	}

	private int launch(File workingDir, String[] vars, String[] parms,
			File output) throws IOException {

		// TODO - should make configurable

		for (int i = 0; i < parms.length; i++) {
			reporter.report(suffix+" init parm " + i + " =" + parms[i]);
		}
		for (int i = 0; i < vars.length; i++) {
			reporter.report(suffix+" init vars " + i + " =" + vars[i]);
		}
		Runtime r = Runtime.getRuntime();

		Process p;

		// if the env var list is empty we chuck it away so that envs are
		// inheritied
		if (vars != null && vars.length == 0) {
			vars = null;
		}
		p = r.exec(parms, vars, workingDir);

		PrintStream out = null;
		PrintStream err = null;
		FileOutputStream fos = null;

		TextStreamRedirector rdr1=null;
		TextStreamRedirector rdr2=null;
		InputStream stdIn=p.getInputStream();
		InputStream  errIn=p.getErrorStream();
		
		if (output != null) {

			fos = new FileOutputStream(output);
			PrintStream ps = new PrintStream(fos);
			rdr1 = new TextStreamRedirector(stdIn,ps);
	 	    rdr2 = new TextStreamRedirector(errIn,ps);
					
		   
		} else {
			// no specific file set. 
			// if we have an reporter we can use it 
			if(reporter!=null) {
				rdr1 = new TextStreamRedirector(stdIn,reporter);
		 	    rdr2 = new TextStreamRedirector(errIn,reporter);
			}	
			else {
			rdr1 = new TextStreamRedirector(stdIn,System.out);
	 	    rdr2 = new TextStreamRedirector(errIn,System.err);
			}
		}

	   
		rdr1.setPrefix(suffix);
		rdr2.setPrefix(suffix);
		rdr1.start();
		rdr2.start();

		try {
			p.waitFor();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		if (fos != null)
			fos.close();

		return p.exitValue();

	}

	public void launch(File workingDir, List list) throws IOException {
		launch(workingDir, new LinkedList(), list);

	}

	public String getSuffix() {
		return suffix;
	}

	public void setSuffix(String suffix) {
		this.suffix = suffix;
	}

	public void launch(File workingDir, List list, File results)
			throws IOException {
		launch(workingDir, new String[0], (String[]) list
				.toArray(new String[list.size()]), results);

	}
}
