/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tck.harness;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Date;
import java.util.Iterator;

import org.apache.kato.tck.api.PropertyStore;


public class PropertyStoreXMLWriter {

	
	private PropertyStore store=null;
	
	public PropertyStoreXMLWriter(PropertyStore store) {
		this.store=store;
	}
	
	/**
	 * Writes the contents of the store to a properties file keys with null
	 * values are written with blank values
	 * 
	 * @param store
	 * @param outfile
	 * @throws IOException
	 */
	public  void write(File outfile)
			throws IOException {
	
		FileWriter fw = new FileWriter(outfile);
		PrintWriter pw = new PrintWriter(fw);
		
		pw.println("<tckconfig created=\"" + new Date()+"\">");
		
		PropertyStore s=store;
		while(s!=null) {
			writeStoreConfig(s,pw);
			s=s.getParent();
		}
				
		pw.println("</tckconfig>");
		pw.close();
	
	}

	private void writeStoreConfig(PropertyStore pstore,PrintWriter pw) {
		
		pw.println("<store source=\""+pstore.getSource()+"\">");
		
		Iterator keyIterator = pstore.iterator();
		
		while (keyIterator.hasNext()) {
			String key = (String) keyIterator.next();
			pw.print("<key name=\""+key+"\" ");
			
			String resolvedValue = store.getProperty(key);
			String value=pstore.getPropertyAsIs(key);
			if(value!=null) {
				pw.print("value=\""+value+"\" " );
			}
			
			if(resolvedValue!=null) {
				pw.print("resolvedvalue=\""+resolvedValue+"\" " );
			}
	
			pw.println(" />");
			
		}
		pw.println("</store>");
	}

}
