/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tck.harness;

import java.io.File;

public class TCKLauncherConfigurator {
	
	private String[] args=null;
	private String error=null;
	private boolean runTests=true;
	private boolean runDump=true;
	private File testConfigFile = null;
	private File outputLocation = null;
	private File tckConfigFile=null;
	
	public TCKLauncherConfigurator(String[] args) {
		this.args=args;
	}
	public String getErrorMessage() {
		return error;
	}
	public boolean  isValid() {
		
		
		if (args == null || args.length == 0) {
			return false;
		}
		
		Boolean dumponly=null;
		Boolean testonly=null;
		
		
		for (int i = 0; i < args.length; i++) {

			String arg = args[i];
			if (arg.equalsIgnoreCase("-testonly")) {
				if(testonly!=null) {
					error="-testonly can only be specified once";
					return false;
				}
				testonly=new Boolean(true);
			}
			else if (arg.equalsIgnoreCase("-dumponly")) {
				if(dumponly!=null) {
					error="-dumponly can only be specified once";
					return false;
				}
				dumponly=new Boolean(true);
			}
			else if (arg.equalsIgnoreCase("-tckconfig")) {
				if (tckConfigFile != null) {
					error="-tckconfig can only be specified once";
					return false;
				}
				if ((i + 1) >= args.length) {
					error="-tckconfig requires a filename";
					return false;
				}
				i++;
				String nextArg=args[i];
				if(nextArg.startsWith("-")) {
					error="-tckconfig requires a filename";
					return false;
				}
				
				tckConfigFile = new File(nextArg);
				
				if (tckConfigFile.exists()==false) {
					error="testsuite file "+tckConfigFile.getAbsolutePath()+" does not exist";
					return false;
				}
				
				if (tckConfigFile.isDirectory()) {
					error="TCK config file "+tckConfigFile.getAbsolutePath()+" is a directory";
					return false;
				}
				
			}
			else if (arg.equalsIgnoreCase("-output")) { 
				if (outputLocation != null) {
					error="-output can only be specified once";
					return false;
				}
				if ((i + 1) >= args.length) {
					error="-output requires a filename";
					return false;
				}
				i++;
				String nextArg=args[i];
				if(nextArg.startsWith("-")) {
					error="-output requires a filename";
					return false;
				}
				
				outputLocation = new File(nextArg);
				
				if (outputLocation.exists() && outputLocation.isDirectory()) {
					error="output location "
							+ outputLocation.getAbsolutePath()
							+ " is a directory";
					return false;
				}
				
			}
			else if (arg.equalsIgnoreCase("-testconfig")) {
				if (testConfigFile != null) {
					error="-testconfig can only be specified once";
					return false;
				}
				if ((i + 1) >= args.length) {
					error="-testconfig requires a filename";
					return false;
				}
				i++;
				String nextArg=args[i];
				if(nextArg.startsWith("-")) {
					error="-testconfig requires a filename";
					return false;
				}
				testConfigFile = new File(nextArg);
				if (testConfigFile.exists() == false) {
					error="testconfig file "
							+ testConfigFile.getAbsolutePath()
							+ " does not exist";
					return false;
				}
				if (testConfigFile.isFile() == false) {
					error="testconfig file "
							+ testConfigFile.getAbsolutePath()
							+ " is not a file";
					return false;
				}

			} else {
				error="argument " + arg + " is not understood";
				
				return false;
			}
		}
		
		if(testonly==null) testonly=new Boolean(false);
		if(dumponly==null) dumponly=new Boolean(false);
		
		runTests=!dumponly.booleanValue();
		runDump=!testonly.booleanValue();
		
		return true;
	}
	public File getTestConfigFile() {
		return testConfigFile;
	}
	public File getOutputLocation() {
		return outputLocation;
	}
	public boolean isRunTests() {
		return runTests;
	}
	public boolean isRunDump() {
		return runDump;
	}
	public File getTCKConfigFile() {
		return tckConfigFile;
	}
}
