/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tck.results.reporter;


import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.TreeMap;

public  abstract class TCKResultMatrixWriter {

	
	protected File dir=null;
	
	public TCKResultMatrixWriter(File root) {
		this.dir=root;
	}
	
	public final void print(TCKResultMatrix matrix) throws IOException {
		
		init();
		writeIndex(matrix);
		writeDetailFiles(matrix);
		
	}
	
	protected  void writeDetailFiles(TCKResultMatrix matrix) throws IOException {
		
		
	}

	public void init() throws IOException{
	
	}
	
private void writeIndex(TCKResultMatrix matrix) throws IOException {
		
	
		SortKeyProvider provider=matrix.getKeyProvider();
		 
		FileWriter fw=new FileWriter(new File(dir,"index.html"));
		PrintWriter out=new PrintWriter(fw);
		HTMLWriter writer=new HTMLWriter(out);
		writer.writeDocHeader("TCK Results Comparision");
		
		writer.startDetailTable();
		
		// class="Failure"
		
		TreeMap map=new TreeMap();
		
		// how many header rows.
		int depth=matrix.getkeyDepth();
		
		// for each one work out total number of leaf children
		// this gives span of table cell for heading
		
		
		for(int d=0;d<depth;d++) {
			// get values for root headings
			writer.startRow();
			writer.writeTableHeaderCell("");
			
		Key[] keyValues=matrix.getKeys(d);
			
			
			for(int k=0;k<keyValues.length;k++) {
					Key value=keyValues[k];
				int width=matrix.getKeyColumnWidth(value);
				writer.writeSpanningtableHeader(width,value.getName());
				
				
			}
			
			writer.endRow();	
				
		}
		
		// summary
		writer.startRow();
		writer.writeTableCell("");
		
	
		Key[] keyValues=matrix.getKeys(depth-1);
		

		for(int k=0;k<keyValues.length;k++) {
				Key value=keyValues[k];
			String background="";
			
			String fails="";
			
			int testcases=matrix.getTestcaseCount(value);
			int failures=matrix.getTestcaseFailureCount(value);
			int errors=matrix.getTestcaseErrorCount(value);
			int bad=failures+errors;
			if(bad>0) {
				background="class=\"Failure\" ";
				
				fails=" ("+bad+")";
			}
			String data=""+testcases+fails;
			if(bad>0) {
				
				data="<a href=\""+value.getPath()+"\">"+data+"</a>";
			}
			
			out.print("<td "+background+">"+data+"</td>");
		}
		
		// get leaf node width for each value
		
		writer.endRow();	
		
		String[] rowNames=matrix.getRowNames();
		for(int r=0;r<rowNames.length;r++) {
		   String rowName=rowNames[r];
			writer.startRow();
			writer.writeTableCell(rowName);
			
			for(int k=0;k<keyValues.length;k++) {
				Key value=keyValues[k];
				String fails="";
				TestcaseResult testresult=matrix.getTestcaseResult(value, rowName);
				String result="";
				if(testresult!=null) {
				 result=testresult.getType();
				}
				String imageName=null;
				
				if(result.equals("failed")) {
					fails="class=\"Failure\" ";
					imageName="resources/purple.png";
					
				} else if(result.equals("error")) {
					fails="class=\"Error\" ";
					imageName="resources/red.png";
				}
				else if(result.equals("passed")) {
					imageName="resources/blue.png";
				}
				String link=result;
				TCKResultSet parentSet=matrix.getResultSet(value,rowName);
				if(parentSet!=null) {
					TestExecutionConfig config=parentSet.getConfig();
					
					
					String key=provider.getkey(config);
					String path="detail/"+key+".html#"+rowName;
					if(imageName!=null) {
						link=writer.createImage(imageName);
						link=writer.createLink(path, link);
					}
					else {
						link=writer.createLink(path,result);
					}
				}
				out.print("<td " +fails+ ">"+link+"</td>");
			}
			writer.endRow();
		}
		
		writer.writeEndTable();
		
		writer.writeDocEnd();
		
		
		
		out.close();
	}

}
