/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tck.scenario142.javaruntime;

import org.apache.kato.tck.harness.scenario.ScenarioElement;




/**
 * Sets up an object containing references to an array of each type.
 * Then finds it in DTFJ and the compares the contents of the arrays with
 * the contents of the arrays from DTFJ using JavaObject.arraycopy()
 * 
 * TODO testcase should be expanded to test the boundary conditions, and expected error conditions.
 * 
 */
public class SetupJavaObject_arraycopy extends ScenarioElement  {

	/**
	 * A container and initializer for each array's contents 
	 * 
	 */
	public static class TestArrays {
		public boolean[] booleanArray;
		public byte[] byteArray;
		public short[] shortArray;
		public char[] charArray;
		public int[] intArray;
		public long[] longArray;
		public float[] floatArray;
		public double[] doubleArray;
		public Integer[] integerArray;
		public Object nonArray = new Object();
		public short[][] arrayArray; 
		
		public TestArrays(int count) {
			booleanArray = new boolean[count];
			byteArray = new byte[count];
			shortArray = new short[count];
			charArray = new char[count];
			intArray = new int[count];
			longArray = new long[count];
			floatArray = new float[count];
			doubleArray = new double[count];
			integerArray = new Integer[count];
			arrayArray = new short[count][count];
			
			for (int i=0; i < count; i++) {
				booleanArray[i] = (i % 2) == 1;
				byteArray[i] = (byte) i;
				shortArray[i] = (short) i;
				charArray[i] = (char) i;
				intArray[i] = i;
				longArray[i] = i;
				floatArray[i] = i;
				doubleArray[i] = i;
				integerArray[i] = new Integer(i);
				
				for(int j = 0; j < count; j++) {
					arrayArray[i][j] = (short) (i + j);
				}
			}

		}
	}

	// The array sizes.
	public  static final int testArraysElementCount = 10;

	// As this is static, this will exist in both the DTFJ dump and the JUnit testcase. No configure needed.
	public static TestArrays theTestObject = new TestArrays(testArraysElementCount);

	public static final String TestArraysClassName = TestArrays.class.getName().replace('.', '/');

	
	
	
}
