/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tck.scenario142.javaruntime;

import org.apache.kato.tck.harness.scenario.ScenarioElement;




/**
 * 
 * Tests JavaThread.getStackFrames(). Configures a thread with a known stack in
 * the live JVM. Finds the thread, by name, in the dump and then retrieves it's
 * JavaStackFrames and checks that the expected methods are there in the correct
 * order. Ignores possible spurious methods inbetween the methods it does expect
 * for implementation specifics.
 * 
 */
public class SetupJavaThread_getStackFrames extends ScenarioElement  {
	public static final String threadName = "TestJavaThread_getStackFrames Thread";

	public static  String[] methodNames = { "wait", "methodC", "methodB", "methodA",
			"run" };

	public static class ContrivedThread extends Thread {
		volatile private boolean running = false;

		public boolean isRunning() {
			return running;
		}

		public ContrivedThread(String name) {
			super(name);
		}

		public void run() {
			methodA(1);
		}

		public int methodA(int arg) {
			return methodB(arg) + 1;
		}

		public int methodB(int arg) {
			return methodC(arg, arg);
		}

		public synchronized int methodC(int arg1, int arg2) {
			int total = arg1 + arg2;

			this.running = true;

			this.notify();
			try {
				wait();
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			return total;
		}
	}

	public void configureThreadStack() {
		ContrivedThread thread = new ContrivedThread(threadName);
		synchronized (thread) {
			thread.start();
			while (!thread.isRunning()) {
				try {
					thread.wait();
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	}

	//public JavaThread contrivedThread;
}
