/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tck.scenario142.javaruntime;

import javax.tools.diagnostics.runtime.java.JavaField;
import javax.tools.diagnostics.runtime.java.JavaObject;

import org.apache.kato.tck.harness.TCKJavaRuntimeTestcase;
import org.apache.kato.tck.scenario142.javaruntime.SetupJavaField_equals;


public class TestJavaField_equals extends TCKJavaRuntimeTestcase {

	
	
	static SetupJavaField_equals.DifferentClass differentObject = new SetupJavaField_equals.DifferentClass();
	
	String differentClassName = SetupJavaField_equals.DifferentClass.class.getName().replace('.', '/');
	
	static JavaObject thisObject = null;	
	public JavaObject getScenerioReference() {
		if (thisObject == null) {
			thisObject = super.getScenerioReference();
		}	
		return thisObject;		
	}
	
	static JavaObject differentClassObject = null;
	public void setUp() {
		if (differentClassObject == null) {
			differentClassObject = getJavaObjectByClassname(differentClassName);
		}
		
		assertNotNull("Unable to find class "+differentClassName, differentClassObject);
	}
	
	public void testToString() throws Exception {
		JavaField fieldAField = getJavaField(getScenerioReference(), "fieldA");
		String theString=fieldAField.toString();
		assertNotNull(theString);
		assertTrue(theString.length() >= 0 );
	}

	public void testHashCode() throws Exception {
		JavaField fieldAField = getJavaField(getScenerioReference(), "fieldA");
		JavaField fieldAField2 = getJavaField(getScenerioReference(), "fieldA");
		
		assertTrue(fieldAField.hashCode()==fieldAField2.hashCode());
	}
	
	
	public void testNullFalse() throws Exception {
		JavaField fieldAField = getJavaField(getScenerioReference(), "fieldA");
		
		assertFalse(fieldAField.equals(null));
	}
	
	public void testNullFalseStatic() throws Exception {
		JavaField staticField = getJavaField(getScenerioReference(), "staticField");
		
		assertFalse(staticField.equals(null));
	}
	
	public void testDifferentFalse() throws Exception {
		JavaField fieldAField = getJavaField(getScenerioReference(), "fieldA");
		JavaField fieldBField = getJavaField(getScenerioReference(), "fieldB");
		
		assertFalse(fieldAField.equals(fieldBField));
		assertFalse(fieldBField.equals(fieldAField));		
	}
	
	public void testDifferentFalseStatic() throws Exception {
		JavaField staticField = getJavaField(getScenerioReference(), "staticField");
		JavaField fieldBField = getJavaField(getScenerioReference(), "fieldB");
		
		assertFalse(staticField.equals(fieldBField));
		assertFalse(fieldBField.equals(staticField));		
	}
	
	public void testEqualsSelf() throws Exception {
		JavaField fieldAField = getJavaField(getScenerioReference(), "fieldA");
		assertTrue(fieldAField.equals(fieldAField));
	}
	
	public void testNotEqualsDiffClass() throws Exception {
		JavaField differentFieldAField = getJavaField(differentClassObject, "fieldA");
		JavaField fieldAField = getJavaField(getScenerioReference(), "fieldA");
		
		assertFalse(fieldAField.equals(differentFieldAField));
		assertFalse(differentFieldAField.equals(fieldAField));
	}
	
	public void testNotEqualsDiffClassStatic() throws Exception {
		JavaField differentStaticField = getJavaField(differentClassObject, "staticField");
		JavaField staticField = getJavaField(getScenerioReference(), "staticField");
		
		assertFalse(staticField.equals(differentStaticField));
		assertFalse(differentStaticField.equals(staticField));
	}

}
