/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tck.scenario142.javaruntime;

import javax.tools.diagnostics.runtime.java.JavaField;
import javax.tools.diagnostics.runtime.java.JavaObject;

import org.apache.kato.tck.harness.TCKJavaRuntimeTestcase;
import org.apache.kato.tck.scenario142.javaruntime.SetupJavaField_getBoolean;


/**
 * Tests that JavaField.getBoolean works as expected.
 *
 */
public class TestJavaField_getBoolean extends TCKJavaRuntimeTestcase {
	
	SetupJavaField_getBoolean setup=new SetupJavaField_getBoolean();
	
	static JavaObject thisObject = null;	
	public JavaObject getScenerioReference() {
		if (thisObject == null) {
			thisObject = super.getScenerioReference();
		}	
		return thisObject;		
	}
	
	public void testIncompatibleClass() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField trueBooleanField = getJavaField(thisJavaObject, "trueBoolean");
		JavaObject incompatibleClass = getJavaObjectByClassname(setup.incompatibleClassName);
		
		assertNotNull("Couldn't find "+setup.incompatibleClassName,incompatibleClass);
		try{
			boolean trueBooleanValue = trueBooleanField.getBoolean(incompatibleClass);
			fail("JavaField.getBoolean() didn't fail when passed Object of wrong class");
		}catch (IllegalArgumentException e) {
			// expected
		}
	}
	
	
	public void testNullObjectInstanceField() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField trueBooleanField = getJavaField(thisJavaObject, "trueBoolean");
		
		try{
			boolean trueBooleanValue = trueBooleanField.getBoolean(null);
			fail("JavaField.getBoolean(null) on instance field didn't throw null pointer exception.");
		}catch(NullPointerException e) {
			// expected.
		}
				
	}

	public void testNonNullObjectInstanceStaticField() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField trueBooleanField = getJavaField(thisJavaObject, "trueStaticBoolean");
		
		boolean trueBooleanValue = trueBooleanField.getBoolean(thisJavaObject);
		
		assertEquals(SetupJavaField_getBoolean.trueStaticBoolean, trueBooleanValue);		
	}
	
	
	public void testTrueBoolean() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField trueBooleanField = getJavaField(thisJavaObject, "trueBoolean");
		
		boolean trueBooleanValue = trueBooleanField.getBoolean(thisJavaObject);
		
		assertEquals(setup.trueBoolean, trueBooleanValue);		
	}
	
	public void testFalseBoolean() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField falseBooleanField = getJavaField(thisJavaObject, "falseBoolean");
		
		boolean falseBooleanValue = falseBooleanField.getBoolean(thisJavaObject);
		
		assertEquals(SetupJavaField_getBoolean.falseStaticBoolean, falseBooleanValue);		
	}
	
	public void testTrueStaticBoolean() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField trueBooleanField = getJavaField(thisJavaObject, "trueStaticBoolean");
		
		boolean trueBooleanValue = trueBooleanField.getBoolean(null);
		
		assertEquals(SetupJavaField_getBoolean.trueStaticBoolean, trueBooleanValue);		
	}
	
	public void testFalseStaticBoolean() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField falseBooleanField = getJavaField(thisJavaObject, "falseStaticBoolean");
		
		boolean falseBooleanValue = falseBooleanField.getBoolean(null);
		
		assertEquals(SetupJavaField_getBoolean.falseStaticBoolean, falseBooleanValue);		
	}
	
	public void testNotBoolean() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField notBooleanField = getJavaField(thisJavaObject, "notBoolean");
		
		try{
			boolean notBooleanValue = notBooleanField.getBoolean(thisJavaObject);
			fail("getBoolean did't throw IllegalArgumentException when passed int field");
		} catch (IllegalArgumentException e) {
			// ok.
		}
				
	}
	
	public void testNotBoolean2() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField notBooleanField = getJavaField(thisJavaObject, "notBoolean2");
		
		try{
			boolean notBooleanValue = notBooleanField.getBoolean(thisJavaObject);
			fail("getBoolean did't throw IllegalArgumentException when passed Object field");
		} catch (IllegalArgumentException e) {
			// ok.
		}
				
	}
}
