/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tck.scenario142.javaruntime;

import java.lang.reflect.Modifier;

import javax.tools.diagnostics.runtime.java.JavaField;
import javax.tools.diagnostics.runtime.java.JavaObject;

import org.apache.kato.tck.harness.TCKJavaRuntimeTestcase;


public class TestJavaField_getModifiers extends TCKJavaRuntimeTestcase {


	static JavaObject thisObject = null;	
	public JavaObject getScenerioReference() {
		if (thisObject == null) {
			thisObject = super.getScenerioReference();
		}	
		return thisObject;
		
	}

	public void testFinalField() throws Exception {
		JavaField field = getJavaField(getScenerioReference(), "finalField");
		assertEquals(Modifier.FINAL, field.getModifiers() & 0xFF);
	}
	
	public void testPrivateField() throws Exception {
		JavaField field = getJavaField(getScenerioReference(), "privateField");
		assertEquals(Modifier.PRIVATE, field.getModifiers() & 0xFF);
	}	
	
	public void testProtectedField() throws Exception {
		JavaField field = getJavaField(getScenerioReference(), "protectedField");
		assertEquals(Modifier.PROTECTED, field.getModifiers() & 0xFF);
	}
	
	public void testPublicField() throws Exception {
		JavaField field = getJavaField(getScenerioReference(), "publicField");
		assertEquals(Modifier.PUBLIC, field.getModifiers() & 0xFF);
	}
	
	public void testStaticField() throws Exception {
		JavaField field = getJavaField(getScenerioReference(), "staticField");
		assertEquals(Modifier.STATIC, field.getModifiers() & 0xFF);
	}
	
	public void testTransientField() throws Exception {
		JavaField field = getJavaField(getScenerioReference(), "transientField");
		assertEquals(Modifier.TRANSIENT, field.getModifiers() & 0xFF);
	}
	
	public void testVolatileField() throws Exception {
		JavaField field = getJavaField(getScenerioReference(), "volatileField");
		assertEquals(Modifier.VOLATILE, field.getModifiers() & 0xFF);
	}
	
	public void testDefaultField() throws Exception {
		JavaField field = getJavaField(getScenerioReference(), "defaultField");
		assertEquals(0, field.getModifiers() & 0xFF);
	}
	
	public void testStaticPrivateVolatileTransientField() throws Exception {
		JavaField field = getJavaField(getScenerioReference(), "staticPrivateVolatileTransientField");
		assertEquals(Modifier.STATIC| Modifier.PRIVATE| Modifier.VOLATILE| Modifier.TRANSIENT
				, field.getModifiers() & 0xFF);
	}
	public void testFinalProtectedField() throws Exception {
		JavaField field = getJavaField(getScenerioReference(), "finalProtectedField");
		assertEquals( Modifier.FINAL | Modifier.PROTECTED, field.getModifiers() & 0xFF);
	}
	
}
