/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tck.scenario142.javaruntime;


import javax.tools.diagnostics.runtime.java.JavaField;
import javax.tools.diagnostics.runtime.java.JavaObject;

import org.apache.kato.tck.harness.TCKJavaRuntimeTestcase;
import org.apache.kato.tck.scenario142.javaruntime.SetupJavaField_getString;


public class TestJavaField_getString extends TCKJavaRuntimeTestcase {
	
	SetupJavaField_getString setup=new SetupJavaField_getString();
	
	static JavaObject thisObject = null;	
	public JavaObject getScenerioReference() {
		if (thisObject == null) {
			thisObject = super.getScenerioReference();
		}	
		return thisObject;		
	}
	
	/**
	 *  Tests that a long string can be retrieved correctly.
	 *  
	 */
	public void testLongString() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		
		assertNotNull(thisJavaObject);		
		
		JavaField longStringField = getJavaField(thisJavaObject, "longString");		
		
		assertNotNull(longStringField);
		
		String longStringValue = longStringField.getString(thisJavaObject);
		
		assertEquals(setup.longString, longStringValue);
	}
	
	public void testEmptyString() throws Exception {
	JavaObject thisJavaObject = getScenerioReference();
		
		assertNotNull(thisJavaObject);		
		
		JavaField emptyStringField = getJavaField(thisJavaObject, "emptyString");		
		
		assertNotNull(emptyStringField);
		
		String emptyStringValue = emptyStringField.getString(thisJavaObject);
		
		assertEquals(setup.emptyString, emptyStringValue);
		
	}
	
	public void testNullString() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();

		assertNotNull(thisJavaObject);		

		JavaField nullStringField = getJavaField(thisJavaObject, "nullString");		

		assertNotNull(nullStringField);

		String nullStringValue = nullStringField.getString(thisJavaObject);

		assertEquals(setup.nullString, nullStringValue);

	}
	
	public void testForeignString() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();

		assertNotNull(thisJavaObject);		

		JavaField foreignStringField = getJavaField(thisJavaObject, "foreignString");		

		assertNotNull(foreignStringField);

		String foreignStringValue = foreignStringField.getString(thisJavaObject);
		
		assertEquals(setup.foreignString, foreignStringValue);

	}
	
	public void testNullJavaObjectOnInstanceField() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		
		assertNotNull(thisJavaObject);
		
		JavaField longStringField = getJavaField(thisJavaObject, "longString");
		
		assertNotNull(longStringField);
		
		try{
			String string = longStringField.getString(null);
			fail ("Expected NullPointerException. Got \""+string+"\"");
		} catch (NullPointerException e) {
			// passes.
		}
	}
	
	public void testStaticString() throws Exception {		
		JavaObject thisJavaObject = getScenerioReference();
		
		assertNotNull(thisJavaObject);
		
		JavaField staticStringField = getJavaField(thisJavaObject, "staticString");
		
		String staticStringValue = staticStringField.getString(null);
		
		assertEquals(SetupJavaField_getString.staticString, staticStringValue);
	}
	
	public void testNonString() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		
		assertNotNull(thisJavaObject);
		
		JavaField nonStringField = getJavaField(thisJavaObject, "nonString");
		
		try {
			String nonStringValue = nonStringField.getString(thisJavaObject);
			fail("Didn't get illegal argument exception when int field read with getString.");
		} catch(IllegalArgumentException e) {
			// passed - expect exception.
		}
	}
}
