/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tck.scenario142.javaruntime;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.tools.diagnostics.image.CorruptData;
import javax.tools.diagnostics.image.CorruptDataException;
import javax.tools.diagnostics.image.DataUnavailable;
import javax.tools.diagnostics.runtime.java.JavaClass;
import javax.tools.diagnostics.runtime.java.JavaLocation;
import javax.tools.diagnostics.runtime.java.JavaObject;
import javax.tools.diagnostics.runtime.java.JavaReference;
import javax.tools.diagnostics.runtime.java.JavaRuntime;
import javax.tools.diagnostics.runtime.java.JavaStackFrame;
import javax.tools.diagnostics.runtime.java.JavaThread;

import org.apache.kato.tck.api.ICheckpoint;
import org.apache.kato.tck.harness.TCKJavaRuntimeTestcase;
import org.apache.kato.tck.scenario142.javaruntime.SetupJavaRuntime_getHeapRoots.HeapRoot;



public class TestJavaStackFrame_getHeapRoots extends TCKJavaRuntimeTestcase {
	
	
	private JavaStackFrame frame = null;     
	private Map rootsCheckList = new HashMap();

//	public String[] jvmDisableJIT()
//	{
//		System.out.println("Disabling JIT...");	
//		return new String[]{"-Xint"};
//	}


	
	public void setUp () {
		JavaRuntime runtime = getJavaRuntime();
		Iterator threads = runtime.getThreads().iterator();
		JavaThread contrivedThread = null;
		while (threads.hasNext()) {
			Object next = threads.next();
			
			if (next instanceof JavaThread) {
				JavaThread thread = (JavaThread) next;
				try {
					//System.out.println("Thread: " + thread.getName());
					if (SetupJavaStackFrame_getHeapRoots.threadName.equals( thread.getName())) {
//						System.out.println("Thread " + thread.getName() + " found!");
						contrivedThread = thread;
						break;
					}
				} catch (CorruptDataException e) {
					assertNotNull(e.getCorruptData());
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
		if (contrivedThread == null) {
			fail("Couldn't find thread "+SetupJavaStackFrame_getHeapRoots.threadName);
		}

		Iterator frames = contrivedThread.getStackFrames().iterator();
		JavaStackFrame currentFrame = null;
		while (frames.hasNext() && frame == null) {
			Object next = frames.next();

			if (next instanceof JavaStackFrame) {
				currentFrame = (JavaStackFrame) next;
				try {
					JavaLocation location = currentFrame.getLocation();
					String methodName = location.getMethod().getName();
//					System.out.println("Method is: " + methodName + " "+ location.getFilename() + "@" + location.getLineNumber());
					if ("methodA".equals(methodName)) {
						frame = currentFrame;
					}
				} catch (Exception e) {
					//e.printStackTrace();
				}
			}
		}
		
		if (null == frame) {
			fail("Couldn't find stack frame for methodA");
		}
		
		Iterator roots = frame.getHeapRoots().iterator();
		while (roots.hasNext()) {
			Object next = roots.next();
			if (next instanceof CorruptData) {
				continue;
			}
			JavaReference r = (JavaReference)next;
			try {
				Object root = r.getTarget();
				int type = r.getRootType();
				HeapRoot rootObject = new HeapRoot(root, type);
				rootsCheckList.put(rootObject, Boolean.FALSE);
				//System.out.println("Adding " + rootObject + " to rootCheckList" );
			} catch (DataUnavailable du) {
			} catch (CorruptDataException cde) {
				assertNotNull(cde.getCorruptData());
			}			
		}

		
	}


	
	
	private HeapRoot[] searchRootByClassName(Map rootCheckList, String classNameToSearch) {
		return searchRootByClassName(rootCheckList, classNameToSearch, 0);
	}

	
	private HeapRoot[] searchRootByClassName(Map rootCheckList, String classNameToSearch, int type) {
		String className;
		ArrayList roots = new ArrayList();
		Iterator i = rootsCheckList.keySet().iterator();
		while (i.hasNext()) {
			Object next = i.next();
			if (!(next instanceof HeapRoot)) {
				continue;
			}
			HeapRoot rootObject = (HeapRoot)next;
			Object root = rootObject.getRoot();
			int rootType = rootObject.getType();
			if (type != 0 && type != rootType) {
				continue;
			}
			try {
				if (root instanceof JavaObject) {
					className = ((JavaObject)root).getJavaClass().getName();
				} else if (root instanceof JavaClass) {
					className = ((JavaClass)root).getName();
				} else {
					continue;
				}
				if (className.endsWith(classNameToSearch)) {
					//System.out.println("Found root object of class: "+ className + " root type: " + HeapRoot.getRootTypeDescription(rootType));
//					synchronized (root) {
//						rootsCheckList.put(root, Boolean.TRUE);
//					}
					roots.add(rootObject);
				}
			} catch (CorruptDataException e) {
				assertNotNull(e.getCorruptData());
				continue;
			}
		}
		//System.out.println("Root object of type " + classNameToSearch + " not found.");
		HeapRoot[] heapRoots = (HeapRoot[])roots.toArray(new HeapRoot[0]);
		return heapRoots;
	}

	
	public void testJavaLocalRoots() throws Exception {
		HeapRoot[] roots = searchRootByClassName(rootsCheckList, "HeapRootsTestLocalVariable", JavaReference.HEAP_ROOT_STACK_LOCAL);
		assertTrue("Found Local Variable object not reported as root.", roots.length != 0);
	}

	public void testParameterRoots() throws Exception {
		HeapRoot[] roots = searchRootByClassName(rootsCheckList, "HeapRootsTestParam", JavaReference.HEAP_ROOT_STACK_LOCAL);
		assertTrue("Found Local Variable object not reported as root.", roots.length != 0);
	}

}
