/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tck.scenario142.javaruntime;

import java.util.Iterator;

import javax.tools.diagnostics.image.CorruptDataException;
import javax.tools.diagnostics.runtime.java.JavaLocation;
import javax.tools.diagnostics.runtime.java.JavaRuntime;
import javax.tools.diagnostics.runtime.java.JavaStackFrame;
import javax.tools.diagnostics.runtime.java.JavaThread;

import org.apache.kato.tck.harness.TCKJavaRuntimeTestcase;
import org.apache.kato.tck.scenario142.javaruntime.SetupJavaThread_getStackFrames;


/**
 * 
 * Tests JavaThread.getStackFrames(). Configures a thread with a known stack in
 * the live JVM. Finds the thread, by name, in the dump and then retrieves it's
 * JavaStackFrames and checks that the expected methods are there in the correct
 * order. Ignores possible spurious methods inbetween the methods it does expect
 * for implementation specifics.
 * 
 */
public class TestJavaThread_getStackFrames extends TCKJavaRuntimeTestcase {
	public void setUp () {
		JavaRuntime runtime = getJavaRuntime();
		Iterator threads = runtime.getThreads().iterator();
		
		while (threads.hasNext()) {
			Object next = threads.next();
			
			if (next instanceof JavaThread) {
				JavaThread thread = (JavaThread) next;
				
				try {
					if (SetupJavaThread_getStackFrames.threadName.equals( thread.getName())) {
						contrivedThread = thread;
						break;
					}
				} catch (CorruptDataException e) {
					assertNotNull(e.getCorruptData());
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
		
		if (contrivedThread == null) {
			fail("Couldn't find thread "+SetupJavaThread_getStackFrames.threadName);
		}
	}

	private JavaThread contrivedThread;

	public void testGetStackFrames() throws Exception {
		Iterator frames = contrivedThread.getStackFrames().iterator();
		int methodIndex = 0;

		while (frames.hasNext()) {
			Object next = frames.next();

			if (next instanceof JavaStackFrame) {
				JavaStackFrame frame = (JavaStackFrame) next;

				JavaLocation location = frame.getLocation();
				
				if ((methodIndex < SetupJavaThread_getStackFrames.methodNames.length)
						&& SetupJavaThread_getStackFrames.methodNames[methodIndex].equals(location.getMethod().getName())) {
					methodIndex++;
				}

			}
		}

		assertEquals("Didn't find all expected methods name. Couldn't find "
				+ ((methodIndex < SetupJavaThread_getStackFrames.methodNames.length) ? SetupJavaThread_getStackFrames.methodNames[methodIndex] : ""), SetupJavaThread_getStackFrames.methodNames.length, methodIndex);
	}
}
