/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tests.junit;

import java.util.Iterator;
import java.util.Properties;

import javax.tools.diagnostics.image.CorruptData;
import javax.tools.diagnostics.image.CorruptDataException;
import javax.tools.diagnostics.image.DiagnosticException;
import javax.tools.diagnostics.image.DataUnavailable;
import javax.tools.diagnostics.image.ImageProcess;
import javax.tools.diagnostics.image.ImageThread;
import javax.tools.diagnostics.runtime.ManagedRuntime;



public class ImageProcessTest extends AbstractImageTestcase {
	
	ImageProcess _proc=null;
	
	protected void setUp() throws Exception {
		_proc = defaultImageProcess();
		super.setUp();
	}

	/*
	 * Test method for 'javax.tools.diagnostics.image.ImageProcess.getCommandLine()'
	 */
	public void testGetCommandLine()
	{
		try {
			assertNotNull(_proc.getCommandLine());
		} catch (DiagnosticException e) {
			//acceptable
			assertTrue(true);
		}
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImageProcess.getEnvironment()'
	 * 
	 * Ensures that the properties are non-null
	 */
	public void testGetEnvironment()
	{
		try {
			Properties env = _proc.getEnvironment();
			assertNotNull(env);
		} catch (DataUnavailable e) {
			//acceptable
		} catch (CorruptDataException e) {
			//acceptable
		}
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImageProcess.getID()'
	 * 
	 * Ensures that the process ID is non-null and non-empty
	 */
	public void testGetID()
	{
		try {
			String id = _proc.getID();
			assertNotNull(id);
			assertTrue(id.length() > 0);
		} catch (DataUnavailable e) {
			//acceptable
		} catch (CorruptDataException e) {
			//acceptable
		}
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImageProcess.getLibraries()'
	 * 
	 * Make sure that the libraries iterator is non-null
	 */
	public void testGetLibraries()
	{
		try {
			assertNotNull(_proc.getLibraries());
		} catch (DataUnavailable e) {
			//this is considered correct
		} catch (CorruptDataException e) {
			//this is considered correct
		}
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImageProcess.getExecutable()'
	 * 
	 * Make sure that the module is non-null
	 */
	public void testGetExecutable()
	{
		try {
			assertNotNull(_proc.getExecutable());
		} catch (DataUnavailable e) {
			//this will be considered a success
		} catch (CorruptDataException e) {
			//normally we won't be testing corrupt data so this is a failure
			assertTrue(false);
		}
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImageProcess.getThreads()'
	 * 
	 * Ensures that there is at least one thread and it is an instance of ImageThread or CorruptData
	 */
	public void testGetThreads()
	{
		Iterator threads = _proc.getThreads().iterator();
		assertNotNull(threads);
		assertTrue(threads.hasNext());
		Object instance = threads.next();
		assertTrue((instance instanceof ImageThread) || (instance instanceof CorruptData));
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImageProcess.getCurrentThread()'
	 * 
	 * Ensures that anything (even null) is returned or the correct exception is thrown
	 */
	public void testGetCurrentThread()
	{
		try {
			_proc.getCurrentThread();
		} catch (CorruptDataException e) {
			//acceptable
		}
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImageProcess.getRuntimes()'
	 * 
	 * Ensures that at least one runtime is returned and that it is an instance of ManagedRuntime or CorruptData
	 */
	public void testGetRuntimes()
	{
		Iterator runtimes = _proc.getRuntimes().iterator();
		assertNotNull(runtimes);
		assertTrue(runtimes.hasNext());
		Object instance = runtimes.next();
		assertTrue((instance instanceof ManagedRuntime) || (instance instanceof CorruptData));
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImageProcess.getSignalNumber()'
	 * 
	 * Ensures that no unexpected exceptions are thrown
	 */
	public void testGetSignalNumber()
	{
		try {
			_proc.getSignalNumber();
		} catch (DataUnavailable e) {
			//acceptable in the spec
		} catch (CorruptDataException e) {
			//acceptable in the spec
		}
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImageProcess.getSignalName()'
	 * 
	 * Ensures that anything (even null) is returned or the correct exceptions are thrown
	 */
	public void testGetSignalName()
	{
		try {
			_proc.getSignalName();
		} catch (DataUnavailable e) {
			//acceptable
		} catch (CorruptDataException e) {
			//acceptable
		}
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImageProcess.getPointerSize()'
	 * 
	 * This should return 31, 32, or 64
	 */
	public void testGetPointerSize()
	{
		int size = _proc.getPointerSize();
		assertTrue((31 == size) || (32 == size) || (64 == size));
	}
}
