/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tests.junit;

import java.util.Iterator;

import javax.tools.diagnostics.image.CorruptDataException;
import javax.tools.diagnostics.image.ImageRegister;

public class ImageRegisterTest extends AbstractImageTestcase
{
	private ImageRegister _testRegister;

	public  ImageRegister defaultImageRegister()
	{
		// if getRegisters() returns an empty iterator (which is valid)
		// then defaultImageRegisters returns null
		Iterator theR = defaultImageThread().getRegisters().iterator();
		ImageRegister theIR = null;
		while (theR.hasNext()){
			theIR = (ImageRegister) theR.next();
		}
		return theIR;
	}

	protected void setUp() throws Exception
	{
		_testRegister = defaultImageRegister();
		super.setUp();
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImageRegister.getName()'
	 * 
	 * Ensures that the register has a non-null non-empty name
	 */
	public void testGetName()
	{
		// _testRegister == null if the registers are empty 
		// ie the iterator is empty which is valid
		if (null != _testRegister){
			String name = _testRegister.getName();
			assertNotNull(name);
			assertTrue(name.length() > 0);
		}
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImageRegister.toString()'
	 * 
	 */
	public void testToString()
	{
		// _testRegister == null if the registers are empty 
		// ie the iterator is empty which is valid
		if (null != _testRegister){
			String name = null;
			name = _testRegister.toString();

			assertNotNull(name);
			assertTrue(name.length() > 0);
		}
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImageRegister.getValue()'
	 * 
	 * Ensures that we can make the call without unexpected exceptions occurring
	 */
	public void testGetValue()
	{
		// _testRegister == null if the registers are empty 
		// ie the iterator is empty which is valid
		if (null != _testRegister){
			try {
				_testRegister.getValue();
			} catch (CorruptDataException e) {
				//accepted by spec
			}
		}
	}
}
