/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tests.junit;

import javax.tools.diagnostics.image.CorruptDataException;
import javax.tools.diagnostics.image.DataUnavailable;
import javax.tools.diagnostics.image.ImagePointer;
import javax.tools.diagnostics.runtime.java.JavaLocation;
import javax.tools.diagnostics.runtime.java.JavaStackFrame;

public class JavaLocationTest extends AbstractImageTestcase
{
	private JavaLocation _location;
	
	public  JavaLocation defaultLocation()
	{
		try {
			return ((JavaStackFrame)defaultJavaThread().getStackFrames().get(0)).getLocation();
		} catch (CorruptDataException e) {
			//XXX: how should this be handled?
			throw new TestNotImplementedException();
		}
	}
	
	protected void setUp() throws Exception
	{
		_location = defaultLocation();
		super.setUp();
	}

	/**
	 * Test method for 'javax.tools.diagnostics.runtime.java.JavaLocation.getAddress()'
	 * 
	 * Ensures that the address is non-null
	 */
	public void testGetAddress()
	{
		try {
			ImagePointer address = _location.getAddress();
			assertNotNull(address);
		} catch (CorruptDataException e) {
			//acceptable
		}
	}

	/**
	 * Test method for 'javax.tools.diagnostics.runtime.java.JavaLocation.getLineNumber()'
	 * 
	 * Ensures that the line number is >0
	 */
	public void testGetLineNumber()
	{
		try {
			assertTrue(_location.getLineNumber() > 0);
		} catch (DataUnavailable e) {
			//acceptable
		} catch (CorruptDataException e) {
			//acceptable
		}
	}

	/**
	 * Test method for 'javax.tools.diagnostics.runtime.java.JavaLocation.getFilename()'
	 * 
	 * Ensures that the filename is non-null and non-empty
	 */
	public void testGetFilename()
	{
		try {
			String filename = _location.getFilename();
			assertNotNull(filename);
			assertTrue(filename.length() > 0);
		} catch (DataUnavailable e) {
			//acceptable
		} catch (CorruptDataException e) {
			//acceptable
		}
	}

	/**
	 * Test method for 'javax.tools.diagnostics.runtime.java.JavaLocation.getCompilationLevel()'
	 * 
	 * Ensures that the call succeeds
	 */
	public void testGetCompilationLevel()
	{
		try {
			_location.getCompilationLevel();
		} catch (CorruptDataException e) {
			//acceptable
		}
	}

	/**
	 * Test method for 'javax.tools.diagnostics.runtime.java.JavaLocation.getMethod()'
	 * 
	 * Ensures that a non-null method is returned
	 */
	public void testGetMethod()
	{
		try {
			assertNotNull(_location.getMethod());
		} catch (CorruptDataException e) {
			//acceptable
		}
	}

	/**
	 * Verify that the equals call doesn't throw
	 */
	public void testEquals()
	{
		try{
			assertTrue(_location.equals(_location));
		} catch (Throwable t) {
			assertTrue(false);
		}
	}
	
	/**
	 * Verify that hashCode() doesn't throw and returns non-zero (technically zero is ok but it will be 
	 * flagged here to ensure that we aren't doing anything bad to create the hashcode)
	 */
	public void testHashCode()
	{
		try {
			assertTrue(0 != _location.hashCode());
		} catch (Throwable t) {
			assertTrue(false);
		}
	}
	
	public void testToString()
	{
		String theString = _location.toString();
		assertNotNull(theString);
		assertTrue(theString.length() >= 0 );
	}
}
