/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tests.junit;

import java.util.Iterator;

import javax.tools.diagnostics.image.CorruptData;
import javax.tools.diagnostics.image.CorruptDataException;
import javax.tools.diagnostics.runtime.java.JavaReference;
import javax.tools.diagnostics.runtime.java.JavaStackFrame;
import javax.tools.diagnostics.runtime.java.JavaThread;


public class JavaStackFrameTest extends AbstractImageTestcase
{
	private JavaStackFrame _frame = null;
	
	public  JavaStackFrame defaultJavaStackFrame()
	{
		JavaThread thread = defaultJavaThread();
		Iterator it = thread.getStackFrames().iterator();
		assertNotNull(it);
		assertTrue(it.hasNext());
		return (JavaStackFrame) it.next();
	}
	
	protected void setUp() throws Exception
	{
		_frame = defaultJavaStackFrame();
		super.setUp();
	}

	/**
	 * Test method for 'javax.tools.diagnostics.runtime.java.JavaStackFrame.getBasePointer()'
	 * 
	 * Ensures that a non-null base pointer is returned
	 */
	public void testGetBasePointer()
	{
		try {
			assertNotNull(_frame.getBasePointer());
		} catch (CorruptDataException e) {
			assertTrue(false);
		}
	}

	/**
	 * Test method for 'javax.tools.diagnostics.runtime.java.JavaStackFrame.getLocation()'
	 * 
	 * Ensures that a non-null location is returned
	 */
	public void testGetLocation()
	{
		try {
			assertNotNull(_frame.getLocation());
		} catch (CorruptDataException e) {
			assertTrue(false);
		}
	}

	/**
	 * Verify that the equals call doesn't throw
	 */
	public void testEquals()
	{
		try{
			assertTrue(_frame.equals(_frame));
		} catch (Throwable t) {
			assertTrue(false);
		}
	}
	
	/**
	 * Verify that hashCode() doesn't throw and returns non-zero (technically zero is ok but it will be 
	 * flagged here to ensure that we aren't doing anything bad to create the hashcode)
	 */
	public void testHashCode()
	{
		
			assertTrue(0 != _frame.hashCode());
		
	}

	/**
	 * Ensures that we get at least one root Reference object from the JavaStackFrame 
	 * 
	 */
	public void testGetHeapRoots()
	{
		try {
			Iterator it = _frame.getHeapRoots().iterator();
			assertNotNull(it);
			/* some frames may legitimately not have any roots */
			if (it.hasNext()){
				Object first = it.next();
				assertTrue( first instanceof JavaReference || first instanceof CorruptData);
			}
		} catch (Throwable t) {
			assertTrue(false);
		}	
	}
}
