/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tests.junit;

import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.jar.Attributes;
import java.util.jar.Manifest;

import javax.tools.diagnostics.image.ImageFactory;

import junit.framework.TestCase;


public class TestManifestProperties extends TestCase {

	
	private static final String ORG_APACHE_KATO_IMAGE = "org/apache/kato/image/";
	private static final String MANIFEST_ROUTE = "/META-INF/MANIFEST.MF";
	private static String resource="/"+ImageFactory.class.getName().replace('.', '/')+".class";
	private URL getKnownResourceURL() {
		URL knownResourceURL=ImageFactory.class.getResource(resource);
		return knownResourceURL;
	}
	
	
	private Manifest getManifest() throws IOException {
		InputStream in=openManifestStream();
		Manifest m=new Manifest(in);
		in.close();
		return m;
	}

	/**
	 * Locates the manifest for .  This manifest may either
	 * be local to the source code (in on the filesystem) on within
	 * the jar
	 * 
	 * @return manifest inputstream or null 
	 * @throws IOException
	 */
	
	private InputStream openManifestStream() throws IOException {
		
		
		URL knownResourceURL = getKnownResourceURL();
		String u=knownResourceURL.toExternalForm();
		
		String protocol=knownResourceURL.getProtocol();
		String manifestLocation=null;
		
		if(protocol.equals("file")) {
			manifestLocation=u.substring(0,u.length()-resource.length())+"/.."+MANIFEST_ROUTE;
		}
		else {
			manifestLocation=u.substring(0,u.length()-resource.length())+MANIFEST_ROUTE;	
		}
		// create expected manifest location...
		
		
		URL u2=new URL(manifestLocation);
		InputStream  manifestStream=u2.openStream();
		return manifestStream;
	}
	
	public void testHasAValidManifestStructure() throws IOException {
		
		InputStream in=openManifestStream();
		Manifest m=new Manifest(in);
		in.close();
		
	}
	public void testHasAManifest() throws IOException {
		
		InputStream manifestStream = openManifestStream();
		assertNotNull(manifestStream);
		manifestStream.close();
		
		
}
	public void testHasBundleVendor() throws IOException {
		
		Manifest m = getManifest();
		Attributes attrs=m.getMainAttributes();
		Object value=attrs.getValue("Bundle-Vendor");
		assertEquals("Bundle Vendor is incorrect","Apache",value.toString());
		
		
	}
	public void testHasBundleVersion() throws IOException {
	
		InputStream in=openManifestStream();
		Manifest m=new Manifest(in);
		Attributes attrs=m.getMainAttributes();
		in.close();
		Object value=attrs.getValue("Bundle-Version");
		assertEquals("Bundle Version is incorrect","1.3.0.qualifier",value.toString());
		
		
	}
	public void testHasAPINameSection() throws IOException {
		
		Manifest m = getManifest();
		Attributes attrs =m.getAttributes(ORG_APACHE_KATO_IMAGE);
		
		assertNotNull("missing  API Name section ["+ORG_APACHE_KATO_IMAGE+"] in manifest",attrs);
		
	}
public void testHasImplementationTitle() throws IOException {
	
	Manifest m = getManifest();
	Attributes attrs =m.getAttributes(ORG_APACHE_KATO_IMAGE);
	Object value=attrs.getValue("Implementation-Title");
	assertNotNull("Implementation-Title is missing in  ["+ORG_APACHE_KATO_IMAGE+"] section of manifest",value);
	
}


public void testHasImplementationVendor() throws IOException {
	
	Manifest m = getManifest();
	Attributes attrs =m.getAttributes(ORG_APACHE_KATO_IMAGE);
	Object value=attrs.getValue("Implementation-Vendor");
	assertNotNull("Implementation-Vendor is missing in  ["+ORG_APACHE_KATO_IMAGE+"] section of manifest",value);
	
}

public void testHasImplementationVersion() throws IOException {
	
	Manifest m = getManifest();
	Attributes attrs =m.getAttributes(ORG_APACHE_KATO_IMAGE);
	Object value=attrs.getValue("Implementation-Version");
	assertNotNull("Implementation-Version is missing in  ["+ORG_APACHE_KATO_IMAGE+"] section of manifest",value);
	
}

public void testHasSpecificationTitle() throws IOException {
	
	Manifest m = getManifest();
	Attributes attrs =m.getAttributes(ORG_APACHE_KATO_IMAGE);
	Object value=attrs.getValue("Specification-Title");
	assertNotNull("Specification-Title is missing in  ["+ORG_APACHE_KATO_IMAGE+"] section of manifest",value);
	
}
public void testHasSpecificationVendor() throws IOException {
	
	Manifest m = getManifest();
	Attributes attrs =m.getAttributes(ORG_APACHE_KATO_IMAGE);
	Object value=attrs.getValue("Specification-Vendor");
	assertNotNull("Specification-Vendor is missing in  ["+ORG_APACHE_KATO_IMAGE+"] section of manifest",value);
	
}
public void testHasSpecificationVersion() throws IOException {
		
		Manifest m = getManifest();
		Attributes attrs =m.getAttributes(ORG_APACHE_KATO_IMAGE);
		Object value=attrs.getValue("Specification-Version");
		assertNotNull("Specification-Version is missing in  ["+ORG_APACHE_KATO_IMAGE+"] section of manifest",value);
		
	}
public void testHasValidSpecificationVendor() throws IOException {
	
	Manifest m = getManifest();
	Attributes attrs =m.getAttributes(ORG_APACHE_KATO_IMAGE);
	Object value=attrs.getValue("Specification-Vendor");
	assertEquals("Specification-Vendor is incorrect in  ["+ORG_APACHE_KATO_IMAGE+"] section of manifest","Apache",value.toString());
	
}
public void testHasValidSpecificationVersion() throws IOException {
	
	Manifest m = getManifest();
	Attributes attrs =m.getAttributes(ORG_APACHE_KATO_IMAGE);
	Object value=attrs.getValue("Specification-Version");
	assertEquals("Specification-Version is incorrect in  ["+ORG_APACHE_KATO_IMAGE+"] section of manifest","1.3",value.toString());
	
}
public void testHasValidSpecificationTitle() throws IOException {
	
	Manifest m = getManifest();
	Attributes attrs =m.getAttributes(ORG_APACHE_KATO_IMAGE);
	Object value=attrs.getValue("Specification-Title");
	assertEquals("Specification-Title is incorrect in  ["+ORG_APACHE_KATO_IMAGE+"] section of manifest","Diagnostic Tool Framework for Java",value.toString());
	
}
public void testHasValidImplementationVendor() throws IOException {
	
	Manifest m = getManifest();
	Attributes attrs =m.getAttributes(ORG_APACHE_KATO_IMAGE);
	Object value=attrs.getValue("Implementation-Vendor");
	assertEquals("Implementation-Vendor is incorrect in  ["+ORG_APACHE_KATO_IMAGE+"] section of manifest","Apache",value.toString());
	
}

public void testHasValidImplementationVersion() throws IOException {
	
	Manifest m = getManifest();
	Attributes attrs =m.getAttributes(ORG_APACHE_KATO_IMAGE);
	Object value=attrs.getValue("Implementation-Version");
	int buildNo=Integer.parseInt(value.toString());
	assertTrue("Implementation Version is incorrect in  ["+ORG_APACHE_KATO_IMAGE+"] section of manifest",buildNo>0);
	
}
}
