/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.katoview.commands;

import java.util.HashMap;
import java.util.Stack;
import java.io.File;

import javax.tools.diagnostics.image.Image;

import org.apache.kato.katoview.Output;
import org.apache.kato.katoview.commands.helpers.Utils;


public class CdCommand extends Command {
	
	public CdCommand(Output o){
		super(o, "cd", "changes the current working directory, used for log files",
				"parameters: <directory_name>\n\n" +
				"Changes the current working directory to <directory_name>, checking " +
				"to see if it exists and is a directory before making the change.  " +
				"The current working directory is where log files are outputted to; " +
				"a change to the current working directory has no effect on the current " +
				"log file setting because the logging filename is converted to an " +
				"absolute path when set.  Note: to see what the current working " +
				"directory is set to, use the \"pwd\" command.\n"
		);
		child_commands = null;
	}
	
	public void doCommand(Stack args, Image loadedImage, HashMap properties){
		if (args.isEmpty())
		{
			out.error("\"cd\" requires exactly one parameter");
			return;
		}
		
		String path = "";

		while (!args.isEmpty())
		{
			path += (String)args.pop() + " ";
		}
		path = path.substring(0, path.length() - 1);
		
		File newPwd = Utils.absPath(properties, path);
		
		if (!newPwd.isDirectory())
		{
			if (newPwd.isFile())
			{
				out.error("cannot change to specified path because it specifies a file, not a directory");
			}
			else
			{
				out.error("specified path is not a directory or a file; it probably doesn't exist");
			}
		}
		else
		{
			properties.put("pwd", newPwd);
		}
	}
}
