/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.katoview.commands;

import java.util.HashMap;
import java.util.Stack;

import javax.tools.diagnostics.image.Image;

import org.apache.kato.katoview.Output;
import org.apache.kato.katoview.commands.helpers.Utils;


public class FindPtrCommand extends Command{
	
	String pattern;
	
	public FindPtrCommand(Output out){
		super(out, "findptr", "searches memory for the given pointer",
				"parameters: see parameters for \"find\" command\n\n" +
				"the findptr command searches for <pattern> as a pointer in the memory segment from <start_address> to <end_address> (both inclusive), " +
				"and outputs the first <matches_to_display> matching addresses that start at the corresponding <memory_boundary>. " +
				"It also display the next <bytes_to_print> bytes for the last match."); 
		child_commands = null;
	}
	
	public void doCommand(Stack args, Image loadedImage, HashMap properties){
		RootCommand rootCommand = (RootCommand)properties.get(Utils.RootCommand_OBJECT);
		if (null == rootCommand){
			out.error("ERROR - No mapping found for RootCommand object");
			return;
		}
		String argsConcact = Utils.concatArgsFromStack(args);
		//TODO split consecutive <,>
		String[] params = argsConcact.split(",");
		if (!isParametersValid(params)) return;
		if(isLittleEndian())
			pattern = reorderBytes();
		String parameters = getparameters(params);

		out.println("issuing: find" + " " + parameters);
		
		Stack findCommandStack = Utils.constructStackFromString("find" + " " + parameters);
		rootCommand.doCommand(findCommandStack, loadedImage, properties);
		//out.println(pattern);
	}
	
	private String getparameters(String[] params){
		String temp = "0x" + pattern;
		for(int i = 1; i < params.length; i++){
			temp+=(","+params[i]);
		}
		return temp;
	}
	
	private String reorderBytes(){
		if (0 != pattern.length()%2)
			pattern = "0" + pattern;
		String temp = "";
		for (int i = pattern.length()/2 - 1; i >= 0; i--){
			temp+=pattern.substring(i*2,i*2+2);
		}
		return temp;
	}
	
	private boolean isLittleEndian(){
		//TODO implement this method once the API is available
		return true;
	}
	
	private boolean isParametersValid(String[] params){
		if (6 != params.length){
			out.error("insufficient number of parameters");
			return false;
		}
		pattern = params[0]; 
		if (pattern.equals("")) {
			out.error("missing pattern string");
			return false;
		}
		
		if (pattern.startsWith("0x")) {
			pattern = pattern.substring(2);
		}
		return true;
	}
}
