/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.katoview.commands;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Stack;
import java.util.Vector;

import javax.tools.diagnostics.image.Image;

import org.apache.kato.katoview.Output;


public class RootCommand extends Command{

	public RootCommand(Output o){
		super(o, "", "", "");
		
		child_commands = new Vector();
		child_commands.add(new InfoCommand(o));
		child_commands.add(new HexdumpCommand(o));
		child_commands.add(new HeapdumpCommand(o));
		child_commands.add(new PlusCommand(o));
		child_commands.add(new MinusCommand(o));
		child_commands.add(new FindCommand(o));
		child_commands.add(new DeadlockCommand(o));
		child_commands.add(new TomcatCommand(o));
		child_commands.add(new XPathCommand(o));
		child_commands.add(new SetCommand(o));
		child_commands.add(new ShowCommand(o));		
		child_commands.add(new WhatisCommand(o));
		child_commands.add(new CdCommand(o));
		child_commands.add(new PwdCommand(o));
		child_commands.add(new FindNextCommand(o));
		child_commands.add(new FindPtrCommand(o));
		child_commands.add(new HelpCommand(o, this));
		child_commands.add(new XCommand(o));
		child_commands.add(new ExitCommand(o));
		child_commands.add(new QuitCommand(o));
	}
	
	public void printHelp(String indent, int descOffset, Stack currArgs, Stack prevArgs)
	{
		if (currArgs.isEmpty())
		{
			// out of arguments; we should get our children to print their
			//  short descriptions
			out.print("\n");
			Iterator itCommand = child_commands.iterator();
			while (itCommand.hasNext())
			{
				Command command = (Command)itCommand.next();
				
				command.printHelp(indent, descOffset, null, prevArgs);
			}
			out.print("\n");
			return;
		}
		
		// ASSERT: !currArgs.isEmpty()
		//  we've still got arguments left so we should keep on processing
		String arg = (String)currArgs.pop();

		// first, we need to process the "x/" command
		if (arg.equals("x"))
		{
			if (!currArgs.isEmpty())
			{
				out.error("\"x\" command does not have any sub-commands; try \"x/\"");
				return;
			}
			currArgs.push("x/");
		}
		else if (arg.startsWith("x/"))
		{
			String xSuffix = arg.substring(2);
			if (!xSuffix.equals(""))
			{
				currArgs.push(xSuffix);
			}
			currArgs.push("x/");
		}
		else
		{
			currArgs.push(arg);
		}
		
		arg = (String)currArgs.pop();
		
		// ASSERT: !currArgs.isEmpty() && null != child_commands
		//  we've still got arguments left and we're an internal node in the command tree;
		//  thus, we'll pass on the job to the next command down
		Iterator itCommand = child_commands.iterator();
		boolean found = false;
		Command currCommand = null;
		while (itCommand.hasNext() && !found)
		{
			currCommand = (Command)itCommand.next();
			if (currCommand.getName().equals(arg))
				found = true;
		}
		
		if (found)
		{
			prevArgs.push(arg);
			currCommand.printHelp(indent, descOffset, currArgs, prevArgs);
		}
		else
		{
			out.error("\"" + arg + "\" is not a command");
			return;
		}
	}
	
	public void doCommand(Stack args, Image loadedImage, HashMap properties)
	{
		String nextCommand = null;
		
		if (!args.isEmpty())
		{
			nextCommand = (String)args.pop();
		}
		else
		{
			out.error("specified command requires additional subcommand");
			return;
		}
		nextCommand = nextCommand.toLowerCase();
		
		if (nextCommand.equals("x"))
		{
			args.push(null);	// signifies empty set of parameters for "x" command
			args.push("x/");
		}
		else if (nextCommand.startsWith("x/"))
		{
			args.push(nextCommand.substring(2));	// push parameters for "x" command
			args.push("x/");
		}
		else
		{
			args.push(nextCommand);
		}
		super.doCommand(args, loadedImage, properties);
	}
}

