/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.katoview.commands.infocommands;

import java.util.HashMap;
import java.util.Stack;
import java.util.Iterator;

import javax.tools.diagnostics.image.Image;
import javax.tools.diagnostics.image.ImageSection;

import org.apache.kato.katoview.Output;
import org.apache.kato.katoview.commands.Command;
import org.apache.kato.katoview.commands.helpers.Utils;


public class InfoMmapCommand extends Command{
	
	public InfoMmapCommand(Output o){
		super(o, "mmap", "outputs a list of all memory segments in the address space", 
				"parameters: none\n\n" +
				"outputs a list all memory segments (ImageSections) " +
				"in the address space: " +
				"start address and size"
				 );
		child_commands = null;
	}
	
	public void doCommand(Stack args, Image loadedImage, HashMap properties){
		Iterator imageSections = Utils.getAddressSapceSectionInfo(loadedImage);
		while(imageSections.hasNext()){
			ImageSection imageSection = (ImageSection)imageSections.next();
			long startAddress = imageSection.getBaseAddress().getAddress();
			long size = imageSection.getSize();
			out.println("Address: 0x" + Long.toHexString(startAddress) + "   size: 0x" + 
					Long.toHexString(size) + " (" + size + ")");
		}
	}
}
